/*
 * acompaction.cpp
 *
 *  Created on: 2. 11. 2014
 *      Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("Automaton compaction binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to compactify",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		cli::Parser parser ( cli::Lexer ( "execute <#stdin > $inputAutomaton" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		parser = cli::Parser ( cli::Lexer ( "execute automaton::transform::Compaction $inputAutomaton > $outputAutomaton" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		parser = cli::Parser ( cli::Lexer ( "execute $outputAutomaton >#stdout" ) );
		parser.parse ( )->run ( environment );

		if(measure.getValue()) common::Streams::measure << measurements::results() << std::endl;

		return 0;
	} catch (const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch(...) {
		return 127;
	}
}
