/*
 * aconvert.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <fstream>

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"
#include "exception/CommonException.h"

#include <convert/DotConverter.h>
#include <convert/GasTexConverter.h>
#include <convert/TikZConverter.h>

#include <automaton/Automaton.h>
#include <grammar/Grammar.h>
#include <regexp/RegExp.h>
#include <tree/Tree.h>
#include <string/String.h>

void automatonFromString ( std::istream & in, std::ostream & out ) {
	automaton::Automaton automaton = alib::StringDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::XmlDataFactory::toStream ( automaton, out );
}

void grammarFromString ( std::istream & in, std::ostream & out ) {
	grammar::Grammar grammar = alib::StringDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::XmlDataFactory::toStream ( grammar, out );
}

void regexpFromString ( std::istream & in, std::ostream & out ) {
	regexp::RegExp regexp = alib::StringDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::XmlDataFactory::toStream ( regexp, out );
}

void stringFromString ( std::istream & in, std::ostream & out ) {
	string::String string = alib::StringDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::XmlDataFactory::toStream ( string, out );
}

void treeFromString ( std::istream & in, std::ostream & out ) {
	tree::Tree tree = alib::StringDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::XmlDataFactory::toStream ( tree, out );
}

void automatonToString ( std::istream & in, std::ostream & out ) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::StringDataFactory::toStream ( automaton, out );
}

void grammarToString ( std::istream & in, std::ostream & out ) {
	grammar::Grammar grammar = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::StringDataFactory::toStream ( grammar, out );
}

void regexpToString ( std::istream & in, std::ostream & out ) {
	regexp::RegExp regexp = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::StringDataFactory::toStream ( regexp, out );
}

void stringToString ( std::istream & in, std::ostream & out ) {
	string::String string = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::StringDataFactory::toStream ( string, out );
}

void treeToString ( std::istream & in, std::ostream & out ) {
	tree::Tree tree = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	alib::StringDataFactory::toStream ( tree, out );
}

void automatonToDot ( std::istream & in, std::ostream & out ) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	convert::DotConverter::convert ( out, automaton );
}

void automatonToGasTex ( std::istream & in, std::ostream & out ) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	convert::GasTexConverter::convert ( out, automaton );
}

void automatonToTikZ ( std::istream & in, std::ostream & out ) {
	automaton::Automaton automaton = alib::XmlDataFactory::fromStream ( in );

	measurements::end ( );
	measurements::start ( "Output write", measurements::Type::AUXILIARY );

	convert::TikZConverter::convert ( out, automaton );
}

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Convert binary", ' ', "0.01" );

		TCLAP::SwitchArg automaton_from_string ( "", "automaton_from_string", "Convert automaton from string representation" );
		cmd.add ( automaton_from_string );

		TCLAP::SwitchArg automaton_to_string ( "", "automaton_to_string", "Convert automaton to string representation" );
		cmd.add ( automaton_to_string );

		TCLAP::SwitchArg automaton_to_dot ( "", "automaton_to_dot", "Convert automaton to dot representation" );
		cmd.add ( automaton_to_dot );

		TCLAP::SwitchArg automaton_to_gastex ( "", "automaton_to_gastex", "Convert automaton to gastex representation" );
		cmd.add ( automaton_to_gastex );

		TCLAP::SwitchArg automaton_to_tikz ( "", "automaton_to_tikz", "Convert automaton to tikz representation" );
		cmd.add ( automaton_to_tikz );

		TCLAP::SwitchArg grammar_from_string ( "", "grammar_from_string", "Convert grammar from string representation" );
		cmd.add ( grammar_from_string );

		TCLAP::SwitchArg grammar_to_string ( "", "grammar_to_string", "Convert grammar to string representation" );
		cmd.add ( grammar_to_string );

		TCLAP::SwitchArg regexp_from_string ( "", "regexp_from_string", "Convert regexp from string representation" );
		cmd.add ( regexp_from_string );

		TCLAP::SwitchArg regexp_to_string ( "", "regexp_to_string", "Convert regexp to string representation" );
		cmd.add ( regexp_to_string );

		TCLAP::SwitchArg string_from_string ( "", "string_from_string", "Convert string from string representation" );
		cmd.add ( string_from_string );

		TCLAP::SwitchArg string_to_string ( "", "string_to_string", "Convert string to string representation" );
		cmd.add ( string_to_string );

		TCLAP::SwitchArg tree_from_string ( "", "tree_from_string", "Convert tree from string representation" );
		cmd.add ( tree_from_string );

		TCLAP::SwitchArg tree_to_string ( "", "tree_to_string", "Convert tree to string representation" );
		cmd.add ( tree_to_string );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::istream * in = NULL;

		if ( input.isSet ( ) ) {
			if ( input.getValue ( ) == "-" )
				in = & std::cin;
			else
				in = new std::ifstream ( input.getValue ( ) );
		} else {
			in = & std::cin;
		}

		if ( automaton_from_string.getValue ( ) )
			automatonFromString ( * in, std::cout );
		else if ( automaton_to_string.getValue ( ) )
			automatonToString ( * in, std::cout );
		else if ( automaton_to_dot.getValue ( ) )
			automatonToDot ( * in, std::cout );
		else if ( automaton_to_gastex.getValue ( ) )
			automatonToGasTex ( * in, std::cout );
		else if ( automaton_to_tikz.getValue ( ) )
			automatonToTikZ ( * in, std::cout );
		else if ( grammar_from_string.getValue ( ) )
			grammarFromString ( * in, std::cout );
		else if ( grammar_to_string.getValue ( ) )
			grammarToString ( * in, std::cout );
		else if ( regexp_from_string.getValue ( ) )
			regexpFromString ( * in, std::cout );
		else if ( regexp_to_string.getValue ( ) )
			regexpToString ( * in, std::cout );
		else if ( string_from_string.getValue ( ) )
			stringFromString ( * in, std::cout );
		else if ( string_to_string.getValue ( ) )
			stringToString ( * in, std::cout );
		else if ( tree_from_string.getValue ( ) )
			treeFromString ( * in, std::cout );
		else if ( tree_to_string.getValue ( ) )
			treeToString ( * in, std::cout );
		else
			throw exception::CommonException("Invalid format specified");

		if ( input.isSet ( ) && ( input.getValue ( ) != "-" ) )
			delete in;

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cerr << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
