/*
 * Determinize.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "Determinize.h"
#include <automaton/transform/PDAToRHPDA.h>
#include <automaton/transform/RHPDAToPDA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/TM/OneTapeDTM.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace determinize {

DFA<> Determinize::determinize(const automaton::DFA<>& automaton) {
	return automaton;
}

auto DeterminizeDFA = registration::AbstractRegister < Determinize, automaton::DFA < >, const automaton::DFA < > & > ( Determinize::determinize );

DPDA < > Determinize::determinize(const automaton::DPDA < > & automaton) {
	return automaton;
}

auto DeterminizeDPDA = registration::AbstractRegister < Determinize, automaton::DPDA < >, const automaton::DPDA < > & > ( Determinize::determinize );

SinglePopDPDA < > Determinize::determinize(const automaton::SinglePopDPDA < > & automaton) {
	return automaton;
}

auto DeterminizeSinglePopDPDA = registration::AbstractRegister < Determinize, automaton::SinglePopDPDA < >, const automaton::SinglePopDPDA < > & > ( Determinize::determinize );

InputDrivenDPDA < > Determinize::determinize(const automaton::InputDrivenDPDA < > & automaton) {
	return automaton;
}

auto DeterminizeInputDrivenDPDA = registration::AbstractRegister < Determinize, automaton::InputDrivenDPDA < >, const automaton::InputDrivenDPDA < > & > ( Determinize::determinize );

VisiblyPushdownDPDA < > Determinize::determinize(const automaton::VisiblyPushdownDPDA < > & automaton) {
	return automaton;
}

auto DeterminizeVisiblyPushdownDPDA = registration::AbstractRegister < Determinize, automaton::VisiblyPushdownDPDA < >, const automaton::VisiblyPushdownDPDA < > & > ( Determinize::determinize );

RealTimeHeightDeterministicDPDA < > Determinize::determinize(const automaton::RealTimeHeightDeterministicDPDA < > & automaton) {
	return automaton;
}

auto DeterminizeRealTimeHeightDeterministicDPDA = registration::AbstractRegister < Determinize, automaton::RealTimeHeightDeterministicDPDA < >, const automaton::RealTimeHeightDeterministicDPDA < > & > ( Determinize::determinize );

DPDA < > Determinize::determinize(const automaton::NPDA < > & automaton) {
	automaton::RealTimeHeightDeterministicNPDA < > rhpda = automaton::PDAToRHPDA::convert(automaton);
	automaton::RealTimeHeightDeterministicDPDA < > dpda = Determinize::determinize(rhpda);
	return automaton::RHPDAToPDA::convert(dpda);
}

auto DeterminizeNPDA = registration::AbstractRegister < Determinize, automaton::DPDA < >, const automaton::NPDA < > & > ( Determinize::determinize );

OneTapeDTM<> Determinize::determinize(const automaton::OneTapeDTM<>& automaton) {
	return automaton;
}

auto DeterminizeOneTapeDTM = registration::AbstractRegister < Determinize, automaton::OneTapeDTM < >, const automaton::OneTapeDTM < > & > ( Determinize::determinize );

DFTA < > Determinize::determinize(const automaton::DFTA < > & automaton) {
	return automaton;
}

auto DeterminizeDFTA = registration::AbstractRegister < Determinize, automaton::DFTA < >, const automaton::DFTA < > & > ( Determinize::determinize );



auto DeterminizeMultiInitialStateNFA = registration::AbstractRegister < Determinize, automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > >, const automaton::MultiInitialStateNFA < > & > ( Determinize::determinize );
auto DeterminizeNFA = registration::AbstractRegister < Determinize, automaton::DFA < DefaultSymbolType, ext::set < DefaultStateType > >, const automaton::NFA < > & > ( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */

#include "DeterminizeIDPDAPart.cxx"
#include "DeterminizeVPAPart.cxx"
#include "DeterminizeRHDPDAPart.cxx"
#include "DeterminizeNFTAPart.cxx"

