/*
 * ToGrammarRightRG.h
 *
 *  Created on: 8. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_GRAMMAR_RIGHT_RG_H_
#define TO_GRAMMAR_RIGHT_RG_H_

#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <common/createUnique.hpp>

namespace grammar {

namespace convert {

/**
 * Converts left regular grammar to right regular grammar.
 */
class ToGrammarRightRG {
public:
	/**
	 * Performs conversion.
	 * @param grammar Left regular grammar to convert
	 * @return right regular grammar which is equivalent to source left regular grammar.
	 */
	template < class SymbolType >
	static grammar::RightRG < SymbolType > convert(const grammar::LeftRG < SymbolType > & grammar);
};

template < class SymbolType >
grammar::RightRG < SymbolType > ToGrammarRightRG::convert(const grammar::LeftRG < SymbolType > & grammar) {
	// 1.
	SymbolType s = common::createUnique( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::RightRG < SymbolType > rrg( s );

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet() ) {
		rrg.addNonterminalSymbol( nonterminalSymbol );
	}

	rrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	rrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2
	for( const auto & rule : grammar.getRules( ) ) {
		const SymbolType& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.template is<ext::pair<SymbolType, SymbolType>>( ) ) {
				const ext::pair<SymbolType, SymbolType>& rhs = ruleRHS.template get<ext::pair<SymbolType, SymbolType>>();

				SymbolType leftSide = ( rhs.first );
				ext::pair<SymbolType, SymbolType> rightSide = ext::make_pair( rhs.second, lhs );
				rrg.addRule( leftSide, std::move ( rightSide ) );

				if( lhs == grammar.getInitialSymbol( ) ) {
					leftSide = rhs.first;
					SymbolType rightSide2 = rhs.second;
					rrg.addRule( leftSide, rightSide2 );
				}
			} else {
				const SymbolType& rhs = ruleRHS.template get<SymbolType>();

				SymbolType leftSide = rrg.getInitialSymbol( );
				ext::pair<SymbolType, SymbolType> rightSide = ext::make_pair( rhs, lhs );
				rrg.addRule( leftSide, std::move ( rightSide ) );

				if( lhs == grammar.getInitialSymbol( ) ) {
					leftSide = rrg.getInitialSymbol( );
					SymbolType rightSide2 = rhs;
					rrg.addRule( leftSide, rightSide2 );
				}
			}
		}
	}
	return rrg;
}

} /* namespace convert */

} /* namespace grammar */

#endif /* TO_GRAMMAR_RIGHT_RG_H_ */
