/*
 * Rename.h
 *
 *  Created on: Sep 27, 2016
 *      Author: Tomas Pecka
 */

#include "Rename.h"

#include <algorithm>
#include <map>
#include <vector>

#include <hexavigesimal>
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

grammar::RightRG < > Rename::rename ( const grammar::RightRG < > & rrg ) {
	int counter = 0;

	ext::map < DefaultSymbolType, DefaultSymbolType > renamingData;

	for ( auto iter = rrg.getNonterminalAlphabet ( ).begin ( ); iter != rrg.getNonterminalAlphabet ( ).end ( ); iter++ )
		renamingData.insert ( std::make_pair ( * iter, DefaultSymbolType ( ext::toBase26 ( counter++ ) ) ) );

	grammar::RightRG < > result ( renamingData.find ( rrg.getInitialSymbol ( ) )->second );
	result.setGeneratesEpsilon ( rrg.getGeneratesEpsilon ( ) );
	result.setTerminalAlphabet ( rrg.getTerminalAlphabet ( ) );

	for ( auto iter = rrg.getNonterminalAlphabet ( ).begin ( ); iter != rrg.getNonterminalAlphabet ( ).end ( ); iter++ )
		result.addNonterminalSymbol ( renamingData.find ( * iter )->second );

	const auto raw = rrg.getRawRules ( );

	for ( auto iter = raw.begin ( ); iter != raw.end ( ); iter++ )
		for ( const ext::vector < DefaultSymbolType > & rhs : iter->second ) {
			if ( rhs.size ( ) == 0 ) continue; // eps rule in raws...

			ext::vector < DefaultSymbolType > rewrited;

			for ( const DefaultSymbolType & rhs_symbol : rhs ) {
				if ( rrg.getNonterminalAlphabet ( ).count ( rhs_symbol ) > 0 )
					rewrited.push_back ( renamingData.find ( rhs_symbol )->second );
				else
					rewrited.push_back ( rhs_symbol );
			}

			result.addRawRule ( renamingData.find ( iter->first )->second, rewrited );
		}

	return result;
}

auto RenameRightRG = registration::AbstractRegister < Rename, grammar::RightRG < >, const grammar::RightRG < > & > ( Rename::rename );

grammar::LeftRG < > Rename::rename ( const grammar::LeftRG < > & rrg ) {
	int counter = 0;

	ext::map < DefaultSymbolType, DefaultSymbolType > renamingData;

	for ( auto iter = rrg.getNonterminalAlphabet ( ).begin ( ); iter != rrg.getNonterminalAlphabet ( ).end ( ); iter++ )
		renamingData.insert ( std::make_pair ( * iter, DefaultSymbolType ( ext::toBase26 ( counter++ ) ) ) );

	grammar::LeftRG < > result ( renamingData.find ( rrg.getInitialSymbol ( ) )->second );
	result.setGeneratesEpsilon ( rrg.getGeneratesEpsilon ( ) );
	result.setTerminalAlphabet ( rrg.getTerminalAlphabet ( ) );

	for ( auto iter = rrg.getNonterminalAlphabet ( ).begin ( ); iter != rrg.getNonterminalAlphabet ( ).end ( ); iter++ )
		result.addNonterminalSymbol ( renamingData.find ( * iter )->second );

	const auto raw = rrg.getRawRules ( );

	for ( auto iter = raw.begin ( ); iter != raw.end ( ); iter++ )
		for ( const ext::vector < DefaultSymbolType > & rhs : iter->second ) {
			if ( rhs.size ( ) == 0 ) continue; // eps rule in raws...

			ext::vector < DefaultSymbolType > rewrited;

			for ( const DefaultSymbolType & rhs_symbol : rhs ) {
				if ( rrg.getNonterminalAlphabet ( ).count ( rhs_symbol ) > 0 )
					rewrited.push_back ( renamingData.find ( rhs_symbol )->second );
				else
					rewrited.push_back ( rhs_symbol );
			}

			result.addRawRule ( renamingData.find ( iter->first )->second, rewrited );
		}

	return result;
}

auto RenameLeftRG = registration::AbstractRegister < Rename, grammar::LeftRG < >, const grammar::LeftRG < > & > ( Rename::rename );

} /* namespace simplify */

} /* namespace grammar */
