/*
 * Author: Radovan Cerveny
 */

#include "BackwardOracleMatching.h"
#include <stringology/matching/OracleMatcherConstruction.h>
#include <stringology/properties/BackboneLength.h>

#include <string/LinearString.h>
#include <registration/AlgoRegistration.hpp>

namespace stringology {

namespace query {

ext::set < unsigned > BackwardOracleMatching::match ( const string::LinearString < > & subject, const automaton::DFA < > & factorOracle ) {
	ext::set < unsigned > occ;

	size_t patternSize = stringology::properties::BackboneLength::length ( factorOracle );
	size_t subjectSize = subject.getContent ( ).size ( );

	const DefaultStateType failState = DefaultStateType ( -1 );

	size_t posInSubject = 0;

	while ( posInSubject <= subjectSize - patternSize ) {

		DefaultStateType currentState = factorOracle.getInitialState ( );

		size_t posInPattern = patternSize;

		while ( posInPattern > 0 && currentState != failState ) {
			auto transition = factorOracle.getTransitions ( ).find ( { currentState, subject.getContent ( ).at ( posInSubject + posInPattern - 1 ) } );

			if ( transition == factorOracle.getTransitions ( ).end ( ) )
				currentState = failState;
			else
				currentState = transition->second;

			posInPattern--;
		}

		if ( currentState != failState )
			 // Yay, there is match!!!
			occ.insert ( posInSubject );

		posInSubject += posInPattern + 1;
	}

	return occ;
}

auto BackwardOracleMatchingLinearStringLinearString = registration::AbstractRegister < BackwardOracleMatching, ext::set < unsigned >, const string::LinearString < > &, const automaton::DFA < > & > ( BackwardOracleMatching::match );

} /* namespace query */

} /* namespace stringology */
