/*
 * Follow.h
 *
 *  Created on: 9. 6. 2015
 *	  Author: Tomas Pecka
 */

#ifndef FOLLOW_H_
#define FOLLOW_H_

#include <core/multipleDispatch.hpp>
#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>
#include <string/Epsilon.h>
#include <vector>
#include <set>
#include <variant>

namespace grammar {

namespace parsing {

class Follow;

typedef ext::map < DefaultSymbolType, ext::set < ext::variant < DefaultSymbolType, string::Epsilon < > > > > FollowResult1;
typedef alib::SingleDispatch < Follow, FollowResult1, const grammar::GrammarBase & > FollowBase1;

typedef ext::set < ext::variant < DefaultSymbolType, string::Epsilon < > > > FollowResult2;
typedef alib::SingleDispatch < Follow, FollowResult2, const grammar::GrammarBase &, const DefaultSymbolType & > FollowBase2;

class Follow : public FollowBase1, public FollowBase2 {
	template < class T >
	static void follow ( const T & grammar, ext::map < DefaultSymbolType, ext::set < ext::variant < DefaultSymbolType, string::Epsilon < > > > > & followSet );

	static Follow & getInstance ( ) {
		static Follow res;

		return res;
	}

public:
	static FollowResult1 follow ( const grammar::Grammar & grammar );

	static FollowResult2 follow ( const grammar::Grammar & grammar, const DefaultSymbolType & nt );

	template < class T >
	static FollowResult1 follow ( const T & grammar );

	template < class T >
	static FollowResult2 follow ( const T & grammar, const DefaultSymbolType & nt );

	static FollowBase1 & getInstance1 ( ) {
		return getInstance ( );
	}

	static FollowBase2 & getInstance2 ( ) {
		return getInstance ( );
	}

};

} /* namespace parsing */

} /* namespace grammar */

#endif /* FOLLOW_H_ */
