#include "TestUtils.h"

#include <unordered_map>

void printGraphData( const graph::DirectedGraph & graph )
{
	unsigned int id;
	ext::unordered_map<graph::Node, unsigned int> nodes;
	ext::unordered_map<graph::DirectedEdge, unsigned int> edges;
	id = 0;
	for (const graph::Node & node : graph.getNodes()) {
		nodes[ node ] = id;
		id++;
	}
	id = 0;
	for (const graph::DirectedEdge & edge : graph.getEdges()) {
		edges[ edge ] = id;
		id++;
	}

	printf("graph data:\n");
	printf("  node cnt: %lu; edge cnt: %lu\n", graph.nodeCount(), graph.edgeCount());
	printf("  edges:\n");
	for (const graph::DirectedEdge & edge : graph.getEdges())
		printf("    [%u] %u -> %u (%d)\n", edges[edge], nodes[edge.getFromNode()], nodes[edge.getToNode()], graph.getEdgeValue( edge ));
	printf("  successors:\n");
	for (const graph::Node & node : graph.getNodes()) {
		printf("    [%u] ", nodes[node]);
		for (const graph::Node & succ : graph.successors( node ))
			printf("%u, ", nodes[succ]);
		putchar('\n');
	}
	printf("  predecessors:\n");
	for (const graph::Node & node : graph.getNodes()) {
		printf("    [%u] ", nodes[node]);
		for (const graph::Node & pred : graph.predecessors( node ))
			printf("%u, ", nodes[pred]);
		putchar('\n');
	}
}
