#ifndef _CLI_HELP_COMMAND_H_
#define _CLI_HELP_COMMAND_H_

#include <command/Command.h>
#include <environment/Environment.h>

namespace cli {

class HelpCommand : public Command {
	std::unique_ptr < cli::Arg > m_command;

public:
	HelpCommand ( std::unique_ptr < cli::Arg > command ) : m_command ( std::move ( command ) ) {
	}

	virtual Command::Result run ( Environment & environment ) const override {
		std::string command;
		if ( m_command != nullptr )
			command = m_command->eval ( environment );

		if ( command == "execute" ) {
			std::cout << "Execute command executes statements and either prints the result or writes the result to a file" << std::endl;
			std::cout << "The qualification of the type of the result is remembered and when connecting statements together the compatibility with respective param is checked." << std::endl;
			std::cout << "It is possible to use ^ symbol to move the result value of a statement to a parameter of another statement." << std::endl;
			std::cout << "" << std::endl;
			std::cout << "statement_list:" << std::endl;
			std::cout << "statement ( | statement )* - at least one statement followed by a pipe separated sequence of other statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "arg:" << std::endl;
			std::cout << "value  - immediate string value" << std::endl;
			std::cout << "#value - string value from environmet variable" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "param:" << std::endl;
			std::cout << "[^] -                   - a value from the previous result (PreviousResultParam)" << std::endl;
			std::cout << "[^] <( statement_list ) - a statement list serving as a param (StatementParam)" << std::endl;
			std::cout << "[^] ( arg ) param       - a casted parameter, type is given by arg value (CastParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "identifier  - a string param - the value is a immediate string (ImmediateParam)" << std::endl;
			std::cout << "number      - a number param - the value is a immediate integer (ImmediateParam)" << std::endl;
			std::cout << "#identifier - a string param - identified by an environment variable (ValueParam)" << std::endl;
			std::cout << "[^] $arg    - a value from a variable, name is given by arg value (VariableParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "<arg        - a value from a xml file, filename is given by arg value (FileParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "statement:" << std::endl;
			std::cout << "name ( param )* output_spec - a statement with params, name is an immediate value (SingleStatement)" << std::endl;
			std::cout << "( type ) [^] statement      - the result of a statement is casted, type is given by value (CastedStatement)" << std::endl;
			std::cout << "{ :type ( statement ) * }   - creates a set of type given by arg value with content values from individual statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "#identifier   - an immediate value from an environment (ValueStatement)" << std::endl;
			std::cout << "$identifier   - a value from a variable, identifier is an immediate value (VariableStatement)" << std::endl;
			std::cout << "$#identifier  - a value from a variable, identified by an environment variable (VariableStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "< [:type] arg - a value from a xml file, identifier is given by arg value, type of the value can be hinted with optional parameter (FileStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "output_spec:" << std::endl;
			std::cout << ">arg     - a value to a xml file, identifier is given by arg value (ResultFileStatement)" << std::endl;
			std::cout << ">$arg    - a value to a variable, identifier is given by arg value  (ResultVariableStatement)" << std::endl;
			std::cout << ">        - a value is discarded" << std::endl;
			std::cout << "         - an empty output specifier prints the result to the stdout (ResultPrintStatement)" << std::endl;
		} else if ( command == "" ) {
			std::cout << "Simple help for the query language" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "command quit: quits the processor." << std::endl;
			std::cout << "command help: shows this help." << std::endl;
			std::cout << "command introspect: prints available algorithms, algorithm overloads, casts, and datatypes." << std::endl;
			std::cout << "command execute: executes statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "for details use help of individual command" << std::endl;
		} else {
			std::cout << "The command " << command << " either does not exist or does not have a help entry." << std::endl;
		}
		return Command::Result::OK;
	}
};

} /* namespace cli */

#endif /* _CLI_HELP_COMMAND_H_ */
