/*
 * CastRegistry.cpp
 *
 *  Created on: 21. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/CastRegistry.hpp>

#include <core/namingApi.hpp>

#include <algorithm>

namespace abstraction {

std::shared_ptr < abstraction::OperationAbstraction > CastRegistry::getAbstraction ( const std::string & target, const std::string & param ) {
	std::set < std::string > targetTypes;
	if ( alib::namingApi::hasTypes ( target ) )
		targetTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( target ), [ ] ( const ext::type_index & type ) { return ext::to_string ( type ); } );
	else
		targetTypes.insert ( target );

	for ( const std::string & toType : targetTypes ) {
		auto cast = getEntries ( ).find ( std::make_pair ( toType, param ) );
		if ( cast != getEntries ( ).end ( ) ) {
			return cast->second->getAbstraction ( );
		}
	}

	throw exception::CommonException ( "Entry from " + param + " to " + target + " not available." );
}

bool CastRegistry::isNoOp ( const std::string & target, const std::string & param ) {
	std::set < std::string > targetTypes;
	if ( alib::namingApi::hasTypes ( target ) )
		targetTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( target ), [ ] ( const ext::type_index & type ) { return ext::to_string ( type ); } );
	else
		targetTypes.insert ( target );

	for ( const std::string & toType : targetTypes )
		if ( param == toType )
			return true;

	return false;
}

bool CastRegistry::castAvailable ( const std::string & target, const std::string & param ) {
	std::set < std::string > targetTypes;
	if ( alib::namingApi::hasTypes ( target ) )
		targetTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( target ), [ ] ( const ext::type_index & type ) { return ext::to_string ( type ); } );
	else
		targetTypes.insert ( target );

	for ( const std::string & toType : targetTypes ) {
		auto cast = getEntries ( ).find ( std::make_pair ( toType, param ) );
		if ( cast != getEntries ( ).end ( ) )
			return true;
	}

	return false;
}

ext::set < std::string > CastRegistry::listFrom ( const std::string & type ) {
	std::set < std::string > sourceTypes;

	if ( alib::namingApi::hasTypes ( type ) )
		sourceTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( type ), [ ] ( const ext::type_index & raw_type ) { return ext::to_string ( raw_type ); } );
	else
		sourceTypes.insert ( type );

	ext::set < std::string > res;

	for ( const std::pair < const std::pair < std::string, std::string >, std::unique_ptr < Entry > > & entry : getEntries ( ) )
		if ( sourceTypes.count ( entry.first.second ) )
			res.insert ( entry.first.first );

	return res;
}

ext::set < std::string > CastRegistry::listTo ( const std::string & type ) {
	std::set < std::string > targetTypes;

	if ( alib::namingApi::hasTypes ( type ) )
		targetTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( type ), [ ] ( const ext::type_index & raw_type ) { return ext::to_string ( raw_type ); } );
	else
		targetTypes.insert ( type );

	ext::set < std::string > res;

	for ( const std::pair < const std::pair < std::string, std::string >, std::unique_ptr < Entry > > & entry : getEntries ( ) )
		if ( targetTypes.count ( entry.first.first ) )
			res.insert ( entry.first.second );

	return res;
}

ext::set < ext::pair < std::string, std::string > > CastRegistry::list ( ) {
	ext::set < ext::pair < std::string, std::string > > res;

	for ( const std::pair < const std::pair < std::string, std::string >, std::unique_ptr < Entry > > & entry : getEntries ( ) )
		res.insert ( ext::make_pair ( entry.first.first, entry.first.second ) );

	return res;
}

} /* namespace abstraction */
