/*
 * AlgorithmHelper.cpp
 *
 *  Created on: 25. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/Registry.h>
#include <exception/CommonException.h>
#include <abstraction/common/AlgorithmHelper.h>
#include <abstraction/common/CastHelper.h>

namespace abstraction {

std::shared_ptr < abstraction::OperationAbstraction > AlgorithmHelper::eval ( const std::string & name, const ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > & params, std::vector < bool > & moves, AlgorithmCategories::AlgorithmCategory category ) {
	ext::vector < std::string > paramTypes;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : params )
		paramTypes.push_back ( param->getReturnType ( ) );

	std::shared_ptr < abstraction::OperationAbstraction > algo = abstraction::Registry::getAlgorithmAbstraction ( name, paramTypes, category );

	unsigned i = 0;
	ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > casted_params;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : params ) {
		if ( abstraction::Registry::isCastNoOp ( algo->getParamType ( i ), param->getReturnType ( ) ) ) {
			casted_params.push_back ( param );
		} else {
			casted_params.push_back ( abstraction::CastHelper::eval ( param, algo->getParamType ( i ), moves [ i ] ) );
			moves [ i ] = true;
		}
		++ i;
	}

	i = 0;
	for ( const std::shared_ptr < abstraction::OperationAbstraction > & param : casted_params ) {
		if ( ! algo->attachInput ( param, i, moves [ i ] ) )
			throw exception::CommonException ( "Can't connect param at " + ext::to_string ( i ) + " of algorithm " + name + " with result of type " + param->getReturnType ( ) + "." );
		++ i;
	}

	if ( ! algo->eval ( ) )
		throw exception::CommonException ( "Eval of algorithm " + name + " failed." );

	if ( abstraction::Registry::hasDowncast ( algo->getRuntimeReturnType ( ), algo->getReturnType ( ) ) ) {
		std::shared_ptr < abstraction::OperationAbstraction > downcaster = abstraction::Registry::getDowncastAbstraction ( algo->getRuntimeReturnType ( ), algo->getReturnType ( ) );
		if ( ! downcaster->attachInput ( algo, 0, true ) )
			throw exception::CommonException ( "Can't connect param at 0 of downcast of algorithm " + name + " with result of type " + algo->getReturnType ( ) + "." );
		if ( ! downcaster->eval ( ) )
			throw exception::CommonException ( "Eval of downcast of algorithm " + name + " failed." );

		algo = downcaster;
	}

	if ( abstraction::Registry::hasNormalize ( algo->getReturnType ( ) ) ) {
		std::shared_ptr < abstraction::OperationAbstraction > normalized = abstraction::Registry::getNormalizeAbstraction ( algo->getReturnType ( ) );
		if ( ! normalized->attachInput ( algo, 0, true ) )
			throw exception::CommonException ( "Can't connect param at 0 of normalize of algorithm " + name + " with result of type " + algo->getReturnType ( ) + "." );
		if ( ! normalized->eval ( ) )
			throw exception::CommonException ( "Eval of normalize of algorithm " + name + " failed." );

		algo = normalized;
	}

	return algo;
}


} /* namespace abstraction */
