/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_SET_H_
#define OBJECTS_SET_H_

#include <set>
#include <string>
#include <sstream>

#include "ContainerBase.h"
#include "ContainerFeatures.h"

#include <object/UniqueObject.h>
#include <object/AnyObject.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType >
class ObjectsSet final : public ext::set < ElementType >, public ContainerBase {
public:
	explicit ObjectsSet ( ext::set < ElementType > );
	explicit ObjectsSet ( );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsSet & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual alib::ObjectBase * inc ( ) &&;

	typedef ObjectsSet < > normalized_type;
};

template < class ElementType >
ObjectsSet < ElementType >::ObjectsSet(ext::set<ElementType> raw) : ext::set<ElementType>(std::move(raw)) {

}

template < class ElementType >
ObjectsSet < ElementType >::ObjectsSet() : ext::set<ElementType>() {

}

template < class ElementType >
ContainerBase* ObjectsSet < ElementType >::clone() const {
	return new ObjectsSet(*this);
}

template < class ElementType >
ContainerBase* ObjectsSet < ElementType >::plunder() && {
	return new ObjectsSet(std::move(*this));
}

template < class ElementType >
int ObjectsSet < ElementType >::compare(const ObjectsSet& other) const {
	static ext::compare<ext::set<ElementType>> comp;
	return comp ( static_cast < const ext::set < ElementType > & > ( * this ), static_cast < const ext::set < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsSet < ElementType >::operator>>(std::ostream& os) const {
	os << "(ObjectsSet " << static_cast < const ext::set < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsSet < ElementType >::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

template < class ElementType >
alib::ObjectBase* ObjectsSet < ElementType >::inc() && {
	return new alib::UniqueObject(alib::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace alib {

template < >
struct normalize < ext::set < alib::Object > > {
	static ext::set < alib::Object > && eval ( ext::set < alib::Object > && source ) {
		return std::move ( source );
	}
};

template < class ElementType >
struct normalize < ext::set < ElementType > > {
	static ext::set < alib::Object > eval ( ext::set < ElementType > && source ) {
		ext::set < alib::Object > res;

		for ( ElementType && element : ext::make_moveable_set ( source ) )
			res.insert ( alib::Object ( alib::AnyObject < ElementType > ( std::move ( element ) ) ) );

		return res;
	}
};

template < class ElementType >
struct normalize < container::ObjectsSet < ElementType >, typename std::enable_if < ! std::is_same < container::ObjectsSet < ElementType >, container::ObjectsSet < > >::value >::type > {
	static container::ObjectsSet < > eval ( container::ObjectsSet < ElementType > && value ) {
		return container::ObjectsSet < > ( normalize < ext::set < ElementType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace alib */

#endif /* OBJECTS_SET_H_ */
