#ifndef _ALGO_REGISTRATION_HPP_
#define _ALGO_REGISTRATION_HPP_

#include <core/multipleDispatch.hpp>

#include <abstraction/AlgorithmRegistry.hpp>

#include <registration/NormalizationRegistration.hpp>

namespace registration {

template < class Algorithm, class RealReturnType, class ... RealParameterTypeBases >
class OverloadRegister {
public:
	OverloadRegister ( typename Algorithm::template rawOverload < RealReturnType, RealParameterTypeBases ... > callback ) {
		Algorithm::template registerOverload < RealReturnType, RealParameterTypeBases ... > ( callback );
	}
};

template < class Algorithm, class ReturnType, class ... ParameterTypes >
class AbstractRegister {
public:
	template < class ... ParamNames, typename std::enable_if < sizeof ... ( ParamNames ) <= sizeof ... ( ParameterTypes ) >::type * = nullptr >
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ), abstraction::AlgorithmCategories::AlgorithmCategory category, ParamNames ... paramNames ) {
		registration::NormalizationRegister < ReturnType > ( );

		std::array < std::string, sizeof ... ( ParameterTypes ) > parameterNames = { { paramNames ... } };
		for ( unsigned i = sizeof ... ( ParamNames ); i < sizeof ... ( ParameterTypes ); ++i ) {
			parameterNames [ i ] = "arg" + ext::to_string ( i );
		}

		abstraction::AlgorithmRegistry::registerAlgorithm < Algorithm, ReturnType, ParameterTypes ... > ( callback, category, std::move ( parameterNames ) );
	}

	template < class ... ParamNames, typename std::enable_if < sizeof ... ( ParamNames ) <= sizeof ... ( ParameterTypes ) >::type * = nullptr >
	AbstractRegister ( ReturnType ( * callback ) ( ParameterTypes ... ) ) : AbstractRegister ( callback, abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT ) {
	}
};

} /* namespace registration */

#endif // _ALGO_REGISTRATION_HPP_
