/*
 * UniqueSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueSymbol.h"
#include <alphabet/Symbol.h>
#include <object/Object.h>

#include <registration/XmlRegistration.hpp>
#include <primitive/xml/Integer.h>

namespace alib {

alphabet::UniqueSymbol xmlApi < alphabet::UniqueSymbol >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	alphabet::Symbol firstSymbol = alib::xmlApi < alphabet::Symbol >::parse ( input );
	primitive::Integer secondSymbol = alib::xmlApi < primitive::Integer >::parse ( input );

	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return alphabet::UniqueSymbol ( std::move ( firstSymbol ), std::move ( secondSymbol ) );
}

bool xmlApi < alphabet::UniqueSymbol >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < alphabet::UniqueSymbol >::xmlTagName ( ) {
	static std::string xmlTagName = "UniqueSymbol";

	return xmlTagName;
}

void xmlApi < alphabet::UniqueSymbol >::compose ( ext::deque < sax::Token > & output, const alphabet::UniqueSymbol & data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < alphabet::Symbol >::compose ( output, data.getSymbol ( ) );
	alib::xmlApi < primitive::Integer >::compose ( output, data.getId ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace alib */

namespace {

static auto xmlWrite = registration::XmlWriterRegister < alphabet::Symbol, alphabet::UniqueSymbol > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, alphabet::UniqueSymbol > ( );

static auto xmlReader = registration::XmlReaderRegister < alphabet::Symbol, alphabet::UniqueSymbol > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, alphabet::UniqueSymbol > ( );

} /* namespace */
