/*
 * AutomatonBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_BASE_H_
#define AUTOMATON_BASE_H_

#include <object/ObjectBase.h>

#include <core/xmlApi.hpp>

namespace automaton {

/**
 * Represents symbol in an alphabet.
 */
class AutomatonBase : public alib::ObjectBase {
public:
	virtual AutomatonBase* clone() const = 0;
	virtual AutomatonBase* plunder() && = 0;
};

} /* namespace automaton */

namespace alib {

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < automaton::AutomatonBase, Type >::value >::type > {
	static Type parse ( ext::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const ext::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
	}

	static std::string xmlTagName ( ) {
		return Type::getXmlTagName();
	}

	static void compose ( ext::deque < sax::Token > & output, const Type & data ) {
		Type::compose ( output, data );
	}

};

} /* namespace alib */

#endif /* AUTOMATON_BASE_H_ */

