/*
 * FailStateLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef FAIL_STATE_LABEL_H_
#define FAIL_STATE_LABEL_H_

#include "LabelBase.h"
#include "Label.h"
#include <object/Object.h>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class FailStateLabel final : public LabelBase {
public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit FailStateLabel ( );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FailStateLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "FailStateLabel";

		return xmlTagName;
	}

	static FailStateLabel parse ( ext::deque < sax::Token >::iterator & input );

	static void compose ( ext::deque < sax::Token > & out, const FailStateLabel & label );

	virtual LabelBase * inc ( ) &&;

	typedef FailStateLabel normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type FailStateLabel::instance ( ) {
	return -1;
}

template < >
inline std::string FailStateLabel::instance < std::string > ( ) {
	return std::string ( "fail" );
}

template < >
inline FailStateLabel FailStateLabel::instance < FailStateLabel > ( ) {
	return FailStateLabel ( );
}

template < >
inline Label FailStateLabel::instance < Label > ( ) {
	return Label ( FailStateLabel ( ) );
}

template < >
inline alib::Object FailStateLabel::instance < alib::Object > ( ) {
	return alib::Object ( FailStateLabel ( ) );
}

} /* namespace label */

#endif /* FAIL_STATE_LABEL_H_ */
