/*
 * FormalRTESymbolSubst.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_SUBST_SYMBOL_H_
#define FORMAL_RTE_SUBST_SYMBOL_H_

#include "FormalRTESymbol.h"

#include <exception/CommonException.h>
#include <sstream>
#include <alphabet/common/SymbolNormalize.h>

namespace rte {

/**
 * Represents constant symbol (element of alphabet K) in the regular expression. Contains name of the symbol.
 */
template < class SymbolType, class RankType >
class FormalRTESymbolSubst : public FormalRTESymbol < SymbolType, RankType >, public ext::NullaryNode < FormalRTESymbolSubst < SymbolType, RankType > > {
public:
	virtual void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit FormalRTESymbolSubst ( common::ranked_symbol < SymbolType, RankType > symbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbolSubst < SymbolType, RankType > * clone ( ) const override;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbolSubst < SymbolType, RankType > * plunder ( ) && override;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const override;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetF, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const override;

	int compare ( const FormalRTESymbolSubst & ) const;

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const override;

	/**
	 * Returns the string representation of RTE Substitution Symbol.
	 */
	virtual explicit operator std::string ( ) const override;

	FormalRTESymbolSubst < DefaultSymbolType, DefaultRankType > normalizeRaw ( ) && {
		return FormalRTESymbolSubst < DefaultSymbolType, DefaultRankType > ( alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( this->m_symbol ) ) );
	}

	virtual ext::smart_ptr < FormalRTESymbol < DefaultSymbolType, DefaultRankType > > normalizeSymbol ( ) && override {
		return ext::smart_ptr < FormalRTESymbol < DefaultSymbolType, DefaultRankType > > ( std::move ( * this ).normalizeRaw ( ).plunder ( ) );
	}

	virtual ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > normalize ( ) && override {
		return ext::smart_ptr < FormalRTEElement < DefaultSymbolType, DefaultRankType > > ( std::move ( * this ).normalizeSymbol ( ) );
	}
};

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType >::FormalRTESymbolSubst ( common::ranked_symbol < SymbolType, RankType > symbol ) : FormalRTESymbol < SymbolType, RankType > ( symbol ) {
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > * FormalRTESymbolSubst < SymbolType, RankType >::clone ( ) const {
	return new FormalRTESymbolSubst ( * this );
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > * FormalRTESymbolSubst < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTESymbolSubst ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTESymbolSubst < SymbolType, RankType >::compare ( const FormalRTESymbolSubst & other ) const {
	static ext::compare < decltype ( this->getSymbol ( ) ) > comp;

	return comp ( this->getSymbol ( ), other.getSymbol ( ) );
}

template < class SymbolType, class RankType >
void FormalRTESymbolSubst < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(FormalRTESymbolSubst " << " symbol = " << this->getSymbol ( ) << "})";
}

template < class SymbolType, class RankType >
bool FormalRTESymbolSubst < SymbolType, RankType >::testSymbol ( const common::ranked_symbol < SymbolType, RankType > & symbol ) const {
	return symbol == this->getSymbol ( );
}

template < class SymbolType, class RankType >
void FormalRTESymbolSubst < SymbolType, RankType >::computeMinimalAlphabet ( ext::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetF */, ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	alphabetK.insert ( this->getSymbol ( ) );
}

template < class SymbolType, class RankType >
bool FormalRTESymbolSubst < SymbolType, RankType >::checkAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetF */, const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return alphabetK.count ( this->getSymbol ( ) ) > 0;
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */

#endif /* FORMAL_RTE_SUBST_SYMBOL_H_ */
