/*
 * PrefixRankedBarNonlinearPattern.h
 *
 *  Created on: Jul 20, 2016
 *      Author: Jan Travnicek
 */

#ifndef PREFIX_RANKED_BAR_NONLINEAR_PATTERN_H_
#define PREFIX_RANKED_BAR_NONLINEAR_PATTERN_H_

#include <set>
#include <vector>
#include <tree>
#include <sstream>
#include <algorithm>
#include <deque>

#include <sax/FromXMLParserHelper.h>
#include <core/components.hpp>
#include <object/UniqueObject.h>
#include <common/ranked_symbol.hpp>
#include <alphabet/xml/RankedSymbol.h>

#include "../TreeFeatures.h"
#include "../TreeException.h"
#include "../RankedTreeBase.h"
#include "../common/TreeFromXMLParser.h"
#include "../common/TreeToXMLComposer.h"
#include "../common/TreeAuxiliary.h"
#include <tree/common/TreeNormalize.h>

#include <alphabet/BarSymbol.h>
#include <alphabet/VariablesBarSymbol.h>
#include <alphabet/SubtreeWildcardSymbol.h>

#include <core/normalize.hpp>

namespace tree {

class GeneralAlphabet;
class SubtreeWildcard;
class BarSymbols;
class VariablesBarSymbol;
class NonlinearAlphabet;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
template < class SymbolType, class RankType >
class PrefixRankedBarNonlinearPattern final : public RankedTreeBase, public alib::Components < PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ext::tuple < GeneralAlphabet, NonlinearAlphabet, BarSymbols >, ext::tuple < SubtreeWildcard, VariablesBarSymbol > > {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > m_Data;

	static ext::vector < common::ranked_symbol < SymbolType, RankType > > toPrefixRankedBar ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & node, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const ext::set < common::ranked_symbol < SymbolType, RankType > > & nonlinearVariables, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar );

public:
	explicit PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables, ext::set < common::ranked_symbol < SymbolType, RankType > > alphabet, ext::vector < common::ranked_symbol < SymbolType, RankType > > data );
	explicit PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables, ext::vector < common::ranked_symbol < SymbolType, RankType > > data );
	explicit PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::vector < common::ranked_symbol < SymbolType, RankType > > data );
	explicit PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedTree < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedPattern < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedNonlinearPattern < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( const PrefixRankedBarTree < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( const PrefixRankedBarPattern < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( const RankedTree < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( const RankedPattern < SymbolType, RankType > & tree );
	explicit PrefixRankedBarNonlinearPattern ( const RankedNonlinearPattern < SymbolType, RankType > & tree );

	virtual RankedTreeBase * clone ( ) const;
	virtual RankedTreeBase * plunder ( ) &&;

	const ext::set < common::ranked_symbol < SymbolType, RankType > > & getAlphabet ( ) const & {
		return this->template accessComponent < GeneralAlphabet > ( ).get ( );
	}

	ext::set < common::ranked_symbol < SymbolType, RankType > > && getAlphabet ( ) && {
		return std::move ( this->template accessComponent < GeneralAlphabet > ( ).get ( ) );
	}

	void extendAlphabet ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & symbols ) {
		this->template accessComponent < GeneralAlphabet > ( ).add ( symbols );
	}

	const ext::set < common::ranked_symbol < SymbolType, RankType > > & getBars ( ) const & {
		return this->template accessComponent < BarSymbols > ( ).get ( );
	}

	ext::set < common::ranked_symbol < SymbolType, RankType > > && getBars ( ) && {
		return std::move ( this->template accessComponent < BarSymbols > ( ).get ( ) );
	}

	const common::ranked_symbol < SymbolType, RankType > & getSubtreeWildcard ( ) const & {
		return this->template accessElement < SubtreeWildcard > ( ).get ( );
	}

	common::ranked_symbol < SymbolType, RankType > && getSubtreeWildcard ( ) && {
		return std::move ( this->template accessElement < SubtreeWildcard > ( ).get ( ) );
	}

	const ext::set < common::ranked_symbol < SymbolType, RankType > > & getNonlinearVariables ( ) const & {
		return this->template accessComponent < NonlinearAlphabet > ( ).get ( );
	}

	ext::set < common::ranked_symbol < SymbolType, RankType > > && getNonlinearVariables ( ) && {
		return std::move ( this->template accessComponent < NonlinearAlphabet > ( ).get ( ) );
	}

	const common::ranked_symbol < SymbolType, RankType > & getVariablesBar ( ) const & {
		return this->template accessElement < VariablesBarSymbol > ( ).get ( );
	}

	common::ranked_symbol < SymbolType, RankType > && getVariablesBar ( ) && {
		return std::move ( this->template accessElement < VariablesBarSymbol > ( ).get ( ) );
	}

	/**
	 * @return List of symbols forming tree (const version).
	 */
	const ext::vector < common::ranked_symbol < SymbolType, RankType > > & getContent ( ) const &;

	ext::vector < common::ranked_symbol < SymbolType, RankType > > && getContent ( ) &&;

	void setContent ( ext::vector < common::ranked_symbol < SymbolType, RankType > > data );

	void arityChecksum ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & data );

	/**
	 * @return true if tree is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrefixRankedBarNonlinearPattern & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName ( ) {
		static std::string xmlTagName = "PrefixRankedBarNonlinearPattern";

		return xmlTagName;
	}

	static PrefixRankedBarNonlinearPattern parse ( ext::deque < sax::Token >::iterator & input );

	static void compose ( ext::deque < sax::Token > & out, const PrefixRankedBarNonlinearPattern & pattern );

	virtual alib::ObjectBase * inc ( ) &&;

	typedef PrefixRankedBarNonlinearPattern < > normalized_type;
};

} /* namespace tree */

#include "PrefixRankedBarTree.h"
#include "PrefixRankedBarPattern.h"
#include "RankedTree.h"
#include "RankedPattern.h"
#include "RankedNonlinearPattern.h"

namespace tree {

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables, ext::set < common::ranked_symbol < SymbolType, RankType > > alphabet, ext::vector < common::ranked_symbol < SymbolType, RankType > > data ) : alib::Components < PrefixRankedBarNonlinearPattern, common::ranked_symbol < SymbolType, RankType >, ext::tuple < GeneralAlphabet, NonlinearAlphabet, BarSymbols >, ext::tuple < SubtreeWildcard, VariablesBarSymbol > > ( ext::make_tuple ( std::move ( alphabet ), std::move ( nonlinearVariables ), std::move ( bars ) ), ext::make_tuple ( std::move ( subtreeWildcard ), std::move ( variablesBar ) ) ) {
	setContent ( std::move ( data ) );
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables, ext::vector < common::ranked_symbol < SymbolType, RankType > > data ) : PrefixRankedBarNonlinearPattern ( bars, variablesBar, subtreeWildcard, nonlinearVariables, ext::set < common::ranked_symbol < SymbolType, RankType > > ( data.begin ( ), data.end ( ) ) + bars + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar, subtreeWildcard } + nonlinearVariables, data ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( ext::set < common::ranked_symbol < SymbolType, RankType > > bars, common::ranked_symbol < SymbolType, RankType > variablesBar, common::ranked_symbol < SymbolType, RankType > subtreeWildcard, ext::vector < common::ranked_symbol < SymbolType, RankType > > data ) : PrefixRankedBarNonlinearPattern ( bars, variablesBar, subtreeWildcard, { }, ext::set < common::ranked_symbol < SymbolType, RankType > > ( data.begin ( ), data.end ( ) ) + bars + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar, subtreeWildcard }, data ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedTree < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar }, variablesBar, alphabet::SubtreeWildcardSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), { }, tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar, alphabet::SubtreeWildcardSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ) }, toPrefixRankedBar ( tree.getContent ( ), alphabet::SubtreeWildcardSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), { }, barBase, variablesBar ) ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedPattern < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar }, variablesBar, tree.getSubtreeWildcard ( ), { }, tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar, tree.getSubtreeWildcard ( ) }, toPrefixRankedBar ( tree.getContent ( ), tree.getSubtreeWildcard ( ), { }, barBase, variablesBar ) ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( SymbolType barBase, common::ranked_symbol < SymbolType, RankType > variablesBar, const RankedNonlinearPattern < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar }, variablesBar, tree.getSubtreeWildcard ( ), tree.getNonlinearVariables ( ), tree.getAlphabet ( ) + TreeAuxiliary::computeBars ( tree.getAlphabet ( ), barBase ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { variablesBar, tree.getSubtreeWildcard ( ) } + tree.getNonlinearVariables ( ), toPrefixRankedBar ( tree.getContent ( ), tree.getSubtreeWildcard ( ), tree.getNonlinearVariables ( ), barBase, variablesBar ) ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( const PrefixRankedBarTree < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( tree.getBars ( ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ) }, alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), alphabet::SubtreeWildcardSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), { }, tree.getAlphabet ( ) + ext::set < common::ranked_symbol < SymbolType, RankType > > { alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), alphabet::SubtreeWildcardSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ) }, tree.getContent ( ) ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( const PrefixRankedBarPattern < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( tree.getBars ( ), tree.getVariablesBar ( ), tree.getSubtreeWildcard ( ), { }, tree.getAlphabet ( ), tree.getContent ( ) ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( const RankedTree < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::instance < SymbolType > ( ), alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), tree ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( const RankedPattern < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::instance < SymbolType > ( ), alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), tree ) {
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::PrefixRankedBarNonlinearPattern ( const RankedNonlinearPattern < SymbolType, RankType > & tree ) : PrefixRankedBarNonlinearPattern ( alphabet::BarSymbol::instance < SymbolType > ( ), alphabet::VariablesBarSymbol::instance < common::ranked_symbol < SymbolType, RankType > > ( ), tree ) {
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > PrefixRankedBarNonlinearPattern < SymbolType, RankType >::toPrefixRankedBar ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const ext::set < common::ranked_symbol < SymbolType, RankType > > & nonlinearVariables, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_structure_iterator iter = tree.structure_begin ( ); iter != tree.structure_end ( ); ++iter )
		if ( iter.getVirtual ( ) )
			if ( ( * iter == subtreeWildcard ) || nonlinearVariables.count ( * iter ) )
				res.push_back ( variablesBar );
			else
				res.push_back ( common::ranked_symbol < SymbolType, RankType > ( barBase, iter->getRank ( ) ) );
		else
			res.push_back ( * iter );

	return res;
}

template < class SymbolType, class RankType >
RankedTreeBase * PrefixRankedBarNonlinearPattern < SymbolType, RankType >::clone ( ) const {
	return new PrefixRankedBarNonlinearPattern ( * this );
}

template < class SymbolType, class RankType >
RankedTreeBase * PrefixRankedBarNonlinearPattern < SymbolType, RankType >::plunder ( ) && {
	return new PrefixRankedBarNonlinearPattern ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
const ext::vector < common::ranked_symbol < SymbolType, RankType > > & PrefixRankedBarNonlinearPattern < SymbolType, RankType >::getContent ( ) const & {
	return this->m_Data;
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > && PrefixRankedBarNonlinearPattern < SymbolType, RankType >::getContent ( ) && {
	return std::move ( this->m_Data );
}

template < class SymbolType, class RankType >
void PrefixRankedBarNonlinearPattern < SymbolType, RankType >::setContent ( ext::vector < common::ranked_symbol < SymbolType, RankType > > data ) {
	arityChecksum ( data );

	ext::set < common::ranked_symbol < SymbolType, RankType > > minimalAlphabet ( data.begin ( ), data.end ( ) );
	ext::set < common::ranked_symbol < SymbolType, RankType > > unknownSymbols;
	std::set_difference ( minimalAlphabet.begin ( ), minimalAlphabet.end ( ), getAlphabet ( ).begin ( ), getAlphabet ( ).end ( ), std::inserter ( unknownSymbols, unknownSymbols.end ( ) ) );

	if ( unknownSymbols.size ( ) > 0 )
		throw TreeException ( "Input symbols not in the alphabet." );

	this->m_Data = std::move ( data );
}

template < class SymbolType, class RankType >
void PrefixRankedBarNonlinearPattern < SymbolType, RankType >::arityChecksum ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & data ) {
	int arityChecksumTerminals = 1;
	int arityChecksumBars = 1;
	int arityChecksumTypes = 0;

	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : data ) {
		if ( getBars ( ).count ( symbol ) || ( symbol == getVariablesBar ( ) ) ) {
			arityChecksumBars += ( unsigned ) symbol.getRank ( );
			arityChecksumBars -= 1;
			arityChecksumTypes -= 1;
		} else {
			arityChecksumTerminals += ( unsigned ) symbol.getRank ( );
			arityChecksumTerminals -= 1;
			arityChecksumTypes += 1;
		}
	}

	if ( ( arityChecksumTerminals != 0 ) || ( arityChecksumBars != 0 ) || ( arityChecksumTypes != 0 ) ) throw TreeException ( "The string does not form a tree" );

	for ( unsigned i = 1; i < data.size ( ); ++i )
		if ( ( data[i - 1] == getSubtreeWildcard ( ) ) && ( data[i] != getVariablesBar ( ) ) )
			throw TreeException ( "Inconsystency of SubtreeWildcard and variablesBar" );

	for ( unsigned i = 1; i < data.size ( ); ++i )
		if ( ( getNonlinearVariables ( ).count ( data[i - 1] ) ) && ( data[i] != getVariablesBar ( ) ) )
			throw TreeException ( "Inconsystency of NonlinearVariables and variablesBar" );

}

template < class SymbolType, class RankType >
bool PrefixRankedBarNonlinearPattern < SymbolType, RankType >::isEmpty ( ) const {
	return this->m_Data.size ( ) == 0;
}

template < class SymbolType, class RankType >
int PrefixRankedBarNonlinearPattern < SymbolType, RankType >::compare ( const PrefixRankedBarNonlinearPattern & other ) const {
	auto first = ext::tie ( m_Data, getAlphabet ( ), getSubtreeWildcard ( ), getNonlinearVariables ( ), getBars ( ), getVariablesBar ( ) );
	auto second = ext::tie ( other.m_Data, other.getAlphabet ( ), other.getSubtreeWildcard ( ), other.getNonlinearVariables ( ), other.getBars ( ), other.getVariablesBar ( ) );

	static ext::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

template < class SymbolType, class RankType >
void PrefixRankedBarNonlinearPattern < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(PrefixRankedBarNonlinearPattern ";

	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : this->m_Data )
		out << symbol;

	out << ")";
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << "\"";

	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : this->m_Data )
		ss << symbol;

	ss << "\"";
	return std::move ( ss ).str ( );
}

template < class SymbolType, class RankType >
PrefixRankedBarNonlinearPattern < SymbolType, RankType > PrefixRankedBarNonlinearPattern < SymbolType, RankType >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, PrefixRankedBarNonlinearPattern::getXmlTagName ( ) );
	ext::set < common::ranked_symbol < SymbolType, RankType > > bars = TreeFromXMLParser::parseBars < common::ranked_symbol < SymbolType, RankType > > ( input );
	common::ranked_symbol < SymbolType, RankType > variablesBarSymbol = TreeFromXMLParser::parseVariablesBar < common::ranked_symbol < SymbolType, RankType > > ( input );
	common::ranked_symbol < SymbolType, RankType > subtreeWildcardSymbol = TreeFromXMLParser::parseSubtreeWildcardSymbol < common::ranked_symbol < SymbolType, RankType > > ( input );
	ext::set < common::ranked_symbol < SymbolType, RankType > > nonlinearVariables = TreeFromXMLParser::parseNonlinearVariables < common::ranked_symbol < SymbolType, RankType > > ( input );
	ext::set < common::ranked_symbol < SymbolType, RankType > > rankedAlphabet = TreeFromXMLParser::parseAlphabet < common::ranked_symbol < SymbolType, RankType > > ( input );
	ext::vector < common::ranked_symbol < SymbolType, RankType > > data = TreeFromXMLParser::parseLinearContent < common::ranked_symbol < SymbolType, RankType > > ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, PrefixRankedBarNonlinearPattern::getXmlTagName ( ) );

	return PrefixRankedBarNonlinearPattern < SymbolType, RankType > ( std::move ( bars ), std::move ( variablesBarSymbol ), std::move ( subtreeWildcardSymbol ), std::move ( nonlinearVariables ), std::move ( rankedAlphabet ), std::move ( data ) );
}

template < class SymbolType, class RankType >
void PrefixRankedBarNonlinearPattern < SymbolType, RankType >::compose ( ext::deque < sax::Token > & out, const PrefixRankedBarNonlinearPattern & pattern ) {
	out.emplace_back ( PrefixRankedBarNonlinearPattern::getXmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	TreeToXMLComposer::composeBars ( out, pattern.getBars ( ) );
	TreeToXMLComposer::composeVariablesBar ( out, pattern.getVariablesBar ( ) );
	TreeToXMLComposer::composeSubtreeWildcard ( out, pattern.getSubtreeWildcard ( ) );
	TreeToXMLComposer::composeNonlinearVariables ( out, pattern.getNonlinearVariables ( ) );
	TreeToXMLComposer::composeAlphabet ( out, pattern.getAlphabet ( ) );
	TreeToXMLComposer::composeContent ( out, pattern.m_Data );
	out.emplace_back ( PrefixRankedBarNonlinearPattern::getXmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType, class RankType >
alib::ObjectBase* PrefixRankedBarNonlinearPattern < SymbolType, RankType >::inc() && {
	return new alib::UniqueObject(alib::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace tree */

namespace alib {

template < class SymbolType, class RankType >
class ComponentConstraint< ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ::tree::GeneralAlphabet > {
public:
	static bool used ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		const ext::vector < common::ranked_symbol < SymbolType, RankType > > & content = pattern.getContent ( );

		return std::find ( content.begin ( ), content.end ( ), symbol ) != content.end ( ) || pattern.template accessElement < ::tree::VariablesBarSymbol > ( ).get ( ) == symbol || pattern.template accessComponent < ::tree::BarSymbols > ( ).get ( ).count ( symbol ) || pattern.template accessElement < ::tree::SubtreeWildcard > ( ).get ( ) == symbol;
	}

	static bool available ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > &, const common::ranked_symbol < SymbolType, RankType > & ) {
		return true;
	}

	static void valid ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > &, const common::ranked_symbol < SymbolType, RankType > & ) {
	}
};

template < class SymbolType, class RankType >
class ComponentConstraint< ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ::tree::BarSymbols > {
public:
	static bool used ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		const ext::vector < common::ranked_symbol < SymbolType, RankType > > & content = pattern.getContent ( );

		return std::find ( content.begin ( ), content.end ( ), symbol ) != content.end ( ) || pattern.template accessElement < ::tree::VariablesBarSymbol > ( ).get ( ) == symbol;
	}

	static bool available ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		return pattern.template accessComponent < ::tree::GeneralAlphabet > ( ).get ( ).count ( symbol );
	}

	static void valid ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > &, const common::ranked_symbol < SymbolType, RankType > & ) {
	}
};

template < class SymbolType, class RankType >
class ComponentConstraint< ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ::tree::NonlinearAlphabet > {
public:
	static bool used ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > &, const common::ranked_symbol < SymbolType, RankType > & ) {
		return false;
	}

	static bool available ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		return pattern.template accessComponent < ::tree::GeneralAlphabet > ( ).get ( ).count ( symbol );
	}

	static void valid ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol) {
		if ( ( unsigned ) symbol.getRank ( ) != 0 )
			throw ::tree::TreeException ( "SubtreeWildcard symbol has nonzero arity" );

		if ( pattern.template accessElement < ::tree::SubtreeWildcard > ( ).get ( ) == symbol )
			throw ::tree::TreeException ( "Symbol " + ext::to_string ( symbol ) + "cannot be set as nonlinear variable since it is already subtree wildcard" );
	}
};

template < class SymbolType, class RankType >
class ElementConstraint< ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ::tree::SubtreeWildcard > {
public:
	static bool available ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		return pattern.template accessComponent < ::tree::GeneralAlphabet > ( ).get ( ).count ( symbol );
	}

	static void valid ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol) {
		if ( ( unsigned ) symbol.getRank ( ) != 0 )
			throw ::tree::TreeException ( "SubtreeWildcard symbol has nonzero arity" );

		if ( pattern.template accessComponent < ::tree::NonlinearAlphabet > ( ).get ( ).count ( symbol ) )
			throw ::tree::TreeException ( "Symbol " + ext::to_string ( symbol ) + "cannot be set as subtree wildcard since it is already nonlinear variable" );
	}
};

template < class SymbolType, class RankType >
class ElementConstraint< ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, common::ranked_symbol < SymbolType, RankType >, ::tree::VariablesBarSymbol > {
public:
	static bool available ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
		return pattern.template accessComponent < ::tree::BarSymbols > ( ).get ( ).count ( symbol );
	}

	static void valid ( const ::tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > &, const common::ranked_symbol < SymbolType, RankType > & symbol) {
		if ( ( unsigned ) symbol.getRank ( ) != 0 )
			throw ::tree::TreeException ( "VariablesBarSymbol has nonzero arity" );
	}
};

template < class SymbolType, class RankType >
struct normalize < tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, typename std::enable_if < ! std::is_same < tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType >, tree::PrefixRankedBarNonlinearPattern < > >::value >::type > {
	static tree::PrefixRankedBarNonlinearPattern < > eval ( tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > && value ) {
		common::ranked_symbol < DefaultSymbolType, DefaultRankType > variablesBars = alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( value ).getVariablesBar ( ) );
		common::ranked_symbol < DefaultSymbolType, DefaultRankType > wildcard = alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( value ).getSubtreeWildcard ( ) );
		ext::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > nonlinearAlphabet = alphabet::SymbolNormalize::normalizeRankedAlphabet ( std::move ( value ).getNonlinearVariables ( ) );
		ext::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > bars = alphabet::SymbolNormalize::normalizeRankedAlphabet ( std::move ( value ).getBars ( ) );
		ext::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > alphabet = alphabet::SymbolNormalize::normalizeRankedAlphabet ( std::move ( value ).getAlphabet ( ) );
		ext::vector < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > content = alphabet::SymbolNormalize::normalizeRankedSymbols ( std::move ( value ).getContent ( ) );

		return tree::PrefixRankedBarNonlinearPattern < > ( std::move ( bars ), std::move ( variablesBars ), std::move ( wildcard ), std::move ( nonlinearAlphabet ), std::move ( alphabet ), std::move ( content ) );
	}
};

} /* namespace alib */

#endif /* PREFIX_RANKED_BAR_NONLINEAR_PATTERN_H_ */
