#include "AutomatonTemplatesTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "automaton/FSM/DFA.h"

#include "automaton/AutomatonException.h"

#include "factory/XmlDataFactory.hpp"

#include <primitive/Integer.h>
#include <primitive/xml/Integer.h>
#include <primitive/Character.h>
#include <primitive/xml/Character.h>

#include <object/Object.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( AutomatonTemplatesTest, "automaton" );
CPPUNIT_TEST_SUITE_REGISTRATION ( AutomatonTemplatesTest );

void AutomatonTemplatesTest::setUp ( ) {
}

void AutomatonTemplatesTest::tearDown ( ) {
}

void AutomatonTemplatesTest::testDFAParser ( ) {
	automaton::DFA < char, int > automaton ( 1 );

	automaton.addState ( 2 );
	automaton.addState ( 3 );
	automaton.addInputSymbol ( 'a' );
	automaton.addInputSymbol ( 'b' );

	automaton.addTransition ( 1, 'a', 2 );
	automaton.addTransition ( 2, 'b', 1 );

	automaton.addFinalState ( 3 );

	CPPUNIT_ASSERT ( automaton == automaton );
	{
		ext::deque < sax::Token > tokens = alib::XmlDataFactory::toTokens ( automaton );
		std::string tmp;
		sax::SaxComposeInterface::printMemory ( tmp, tokens );

		ext::deque < sax::Token > tokens2;
		sax::SaxParseInterface::parseMemory ( tmp, tokens2 );
		automaton::DFA < char, int > automaton2 = alib::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

		CPPUNIT_ASSERT ( automaton == automaton2 );
	}
	{
		ext::deque < sax::Token > tokens = alib::XmlDataFactory::toTokens ( automaton );
		std::string tmp;
		sax::SaxComposeInterface::printMemory ( tmp, tokens );

		ext::deque < sax::Token > tokens2;
		sax::SaxParseInterface::parseMemory ( tmp, tokens2 );
		automaton::DFA < alib::Object, alib::Object > automaton2 = alib::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

		automaton::DFA < alib::Object, alib::Object > automaton3 ( alib::Object ( primitive::Integer ( 1 ) ) );

		automaton3.addState ( alib::Object ( primitive::Integer ( 2 ) ) );
		automaton3.addState ( alib::Object ( primitive::Integer ( 3 ) ) );
		automaton3.addInputSymbol ( alib::Object ( primitive::Character ( 'a' ) ) );
		automaton3.addInputSymbol ( alib::Object ( primitive::Character ( 'b' ) ) );

		automaton3.addTransition ( alib::Object ( primitive::Integer ( 1 ) ), alib::Object ( primitive::Character ( 'a' ) ), alib::Object ( primitive::Integer ( 2 ) ) );
		automaton3.addTransition ( alib::Object ( primitive::Integer ( 2 ) ), alib::Object ( primitive::Character ( 'b' ) ), alib::Object ( primitive::Integer ( 1 ) ) );

		automaton3.addFinalState ( alib::Object ( primitive::Integer ( 3 ) ) );

		CPPUNIT_ASSERT ( automaton2 == automaton3 );
	}
}
