/*
 * TreeFromRawParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include <tree/Tree.h>
#include <tree/RankedTreeWrapper.h>
#include <tree/TreeClasses.h>
#include <alphabet/Symbol.h>
#include <alphabet/BarSymbol.h>
#include "TreeFromRawParser.h"

namespace tree {

Tree TreeFromRawParser::parseTree(ext::deque<sax::Token>::iterator& input) const {
	return parseTree(input, ext::set<FEATURES>({FEATURES::RANKED_TREE, FEATURES::PREFIX_RANKED_TREE, FEATURES::PREFIX_RANKED_BAR_TREE, FEATURES::UNRANKED_TREE}));
}

Tree TreeFromRawParser::parseTree(ext::deque<sax::Token>::iterator& input, const ext::set<FEATURES>& features) const {
	UnrankedTree < alphabet::Symbol > tree(this->parseContent(input));

	if(features.count(FEATURES::UNRANKED_TREE)) return Tree{tree};

	if(features.count(FEATURES::PREFIX_RANKED_TREE)) return Tree{PrefixRankedTree < alphabet::Symbol, primitive::Unsigned > {RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}}};

	if(features.count(FEATURES::PREFIX_RANKED_BAR_TREE)) return Tree{PrefixRankedBarTree < alphabet::Symbol, primitive::Unsigned > {alphabet::Symbol{alphabet::BarSymbol{}}, RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}}};

	if(features.count(FEATURES::RANKED_TREE)) return Tree{RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}};

	throw exception::CommonException("Invalid input");
}

RankedTreeWrapper TreeFromRawParser::parseRankedTree(ext::deque<sax::Token>::iterator& input) const {
	return parseRankedTree(input, ext::set<FEATURES>({FEATURES::RANKED_TREE, FEATURES::PREFIX_RANKED_TREE, FEATURES::PREFIX_RANKED_BAR_TREE}));
}

RankedTreeWrapper TreeFromRawParser::parseRankedTree(ext::deque<sax::Token>::iterator& input, const ext::set<FEATURES>& features) const {
	UnrankedTree < alphabet::Symbol > tree(this->parseContent(input));

	if(features.count(FEATURES::RANKED_TREE)) return RankedTreeWrapper{RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}};

	if(features.count(FEATURES::PREFIX_RANKED_TREE)) return RankedTreeWrapper{PrefixRankedTree < alphabet::Symbol, primitive::Unsigned > {RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}}};

	if(features.count(FEATURES::PREFIX_RANKED_BAR_TREE)) return RankedTreeWrapper{PrefixRankedBarTree < alphabet::Symbol, primitive::Unsigned > {alphabet::Symbol{alphabet::BarSymbol{}}, RankedTree < alphabet::Symbol, primitive::Unsigned > {tree}}};

	throw exception::CommonException("Invalid input");
}

ext::tree < alphabet::Symbol > TreeFromRawParser::parseContent(ext::deque<sax::Token>::iterator& input) const {
	if (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		ext::tree < alphabet::Symbol > node ( alphabet::Symbol(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::START_ELEMENT)), {});
		while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT) || sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER)) {
			node.push_back(node.root( ), this->parseContent(input));
		}
		sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::END_ELEMENT);
		return node;
	} else if(sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER)) {
		return ext::tree < alphabet::Symbol > (alphabet::Symbol(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)), {});
	} else {
		throw exception::CommonException("Invalid token stream");
	}
}

} /* namespace tree */
