/*
 * AutomatonFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonFromStringLexer.h"

namespace automaton {

AutomatonFromStringLexer::Token AutomatonFromStringLexer::next(std::istream& in) {
	AutomatonFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\t') {
		token.raw += character;
		goto L0;
	} else if(character == '\n') {
		token.type = TokenType::NEW_LINE;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '<') {
		token.type = TokenType::OUT;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '>') {
		token.type = TokenType::IN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '|') {
		token.type = TokenType::SEPARATOR;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '-') {
		token.type = TokenType::NONE;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '#') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else if(in.unget(), in >> ext::string ( "MISNFA" ) ) {
		token.type = TokenType::MULTI_INITIAL_STATE_NFA;
		token.value = "MISNFA";
		token.raw = "MISNFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "ENFA" ) ) {
		token.type = TokenType::EPSILON_NFA;
		token.value = "ENFA";
		token.raw = "ENFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "NFA" ) ) {
		token.type = TokenType::NFA;
		token.value = "NFA";
		token.raw = "NFA";
		return token;
	} else if(in.clear(), in >> ext::string ( "DFA" ) ) {
		token.type = TokenType::DFA;
		token.value = "DFA";
		token.raw = "DFA";
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == 'E') {
		token.type = TokenType::EPSILON;
		token.value += character;
		token.raw += character;
		return token;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
}

void AutomatonFromStringLexer::putback(std::istream& in, AutomatonFromStringLexer::Token token) {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace automaton */
