/*
 * ObjectToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectToStringComposer.h"
#include "../StringApi.hpp"
#include <registration/AlgoRegistration.hpp>

namespace alib {

void ObjectToStringComposer::compose ( std::ostream & out, const WrapperBaseBase & object ) {
	WrapperBaseDispatcher::dispatch(out, object);
}

void ObjectToStringComposer::compose ( std::ostream & out, const CommonBaseBase & object ) {
	ObjectBaseDispatcher::dispatch(out, object);
}

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const Object& object) {
	compose( out, object.getData ( ) );
}

auto ObjectToStringComposerObject = registration::OverloadRegister < WrapperBaseDispatcher, void, Object>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Primitive& primitive) {
	compose(out, primitive.getData());
}

auto ObjectToStringComposerPrimitive = registration::OverloadRegister < WrapperBaseDispatcher, void, primitive::Primitive>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& output, const label::Label& label) {
	compose(output, label.getData());
}

auto ObjectToStringComposerLabel = registration::OverloadRegister < WrapperBaseDispatcher, void, label::Label>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::Symbol& symbol) {
	compose(out, symbol.getData());
}

auto ObjectToStringComposerSymbol = registration::OverloadRegister < WrapperBaseDispatcher, void, alphabet::Symbol>(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const Void&) {
	out << "Void";
}

auto ObjectToStringComposerVoid = registration::OverloadRegister < ObjectBaseDispatcher, void, Void>(ObjectToStringComposer::compose);

auto ObjectToStringComposerAnyObjectSet = registration::OverloadRegister < ObjectBaseDispatcher, void, AnyObject < ext::set < Object > > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerAnyObjectPair = registration::OverloadRegister < ObjectBaseDispatcher, void, AnyObject < ext::pair < Object, Object > > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerAnyObjectUnsigned = registration::OverloadRegister < ObjectBaseDispatcher, void, AnyObject < unsigned > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerAnyObjectPairUnsignedUnsigned = registration::OverloadRegister < ObjectBaseDispatcher, void, AnyObject < ext::pair < unsigned, unsigned > > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerObjectsPair = registration::OverloadRegister < ObjectBaseDispatcher, void, container::ObjectsPair < Object, Object > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerObjectsSet = registration::OverloadRegister < ObjectBaseDispatcher, void, container::ObjectsSet < Object > >(ObjectToStringComposer::compose);

auto ObjectToStringComposerObjectsVector = registration::OverloadRegister < ObjectBaseDispatcher, void, container::ObjectsVector < Object > >(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const primitive::String& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerString = registration::OverloadRegister < ObjectBaseDispatcher, void, primitive::String>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Character& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerCharacter = registration::OverloadRegister < ObjectBaseDispatcher, void, primitive::Character>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Integer& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerInteger = registration::OverloadRegister < ObjectBaseDispatcher, void, primitive::Integer>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Unsigned& primitive) {
	out << primitive.getData();
}

auto ObjectToStringComposerUnsigned = registration::OverloadRegister < ObjectBaseDispatcher, void, primitive::Unsigned>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, unsigned primitive) {
	out << primitive;
}

void ObjectToStringComposer::compose(std::ostream& out, const primitive::Bool& primitive) {
	out << (primitive.getData() ? "true" : "false");
}

auto ObjectToStringComposerBool = registration::OverloadRegister < ObjectBaseDispatcher, void, primitive::Bool>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::LabeledSymbol& symbol) {
	compose(out, symbol.getLabel());
}

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

void ObjectToStringComposer::compose(std::ostream& out, const label::PrimitiveLabel& label) {
	compose(out, label.getData());
}

auto ObjectToStringComposerPrimitiveLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::PrimitiveLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::HexavigesimalLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerHexavigesimalLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::HexavigesimalLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::ObjectLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerObjectLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::ObjectLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::UniqueLabel& label) {
	out << '<';
	compose(out, label.getLabel().getData());
	out << ", ";
	compose(out, label.getId());
	out << '>';
}

auto ObjectToStringComposerUniqueLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::UniqueLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::FailStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerFailStateLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::FailStateLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::InitialStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerInitialStateLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::InitialStateLabel>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const label::FinalStateLabel& label) {
	out << (std::string) label;
}

auto ObjectToStringComposerFinalStateLabel = registration::OverloadRegister < ObjectBaseDispatcher, void, label::FinalStateLabel>(ObjectToStringComposer::compose);

// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

auto ObjectToStringComposerLabeledSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::LabeledSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BlankSymbol&) {
	out << "#B";
}

auto ObjectToStringComposerBlankSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::BlankSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BottomOfTheStackSymbol&) {
	out << "#T";
}

auto ObjectToStringComposerBottomOfTheStackSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::BottomOfTheStackSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::EndSymbol&) {
	out << "#$";
}

auto ObjectToStringComposerEndSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::EndSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::StartSymbol&) {
	out << "#^";
}

auto ObjectToStringComposerStartSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::StartSymbol>(ObjectToStringComposer::compose);

auto ObjectToStringComposerRankedSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::RankedSymbol < > >(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::BarSymbol&) {
	out << "#|";
}

auto ObjectToStringComposerBarSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::BarSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::VariablesBarSymbol&) {
	out << "#/";
}

auto ObjectToStringComposerVariablesBarSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::VariablesBarSymbol>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::SubtreeWildcardSymbol&) {
	out << "#S";
}

auto ObjectToStringComposerSubtreeWildcardSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::SubtreeWildcardSymbol>(ObjectToStringComposer::compose);

auto ObjectToStringComposerNonlinearVariableSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::NonlinearVariableSymbol < > >(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const alphabet::UniqueSymbol& symbol) {
	out << '<';
	compose(out, symbol.getSymbol().getData());
	out << ", ";
	compose(out, symbol.getId());
	out << '>';
}

auto ObjectToStringComposerUniqueSymbol = registration::OverloadRegister < ObjectBaseDispatcher, void, alphabet::UniqueSymbol>(ObjectToStringComposer::compose);

} /* namespace alib */
