/*
 * GraphToStringComposer.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_TO_STRING_COMPOSER_H_
#define GRAPH_TO_STRING_COMPOSER_H_

#include <set>
#include <core/multipleDispatch.hpp>
#include "graph/GraphClasses.h"

namespace graph {

class GraphToStringComposer : public alib::SingleDispatchFirstStaticParam<GraphToStringComposer, void, std::ostream&, const GraphBase &> {
private:
	static void composeNode(std::ostream& out, const Node &node);
	static void composeEdge(std::ostream& out, const DirectedEdge &edge);
	static void composeEdge(std::ostream& out, const UndirectedEdge &edge);

	static void composeRepresentation(std::ostream &out, REPRESENTATION representation);
	static void composeNodes(std::ostream &out, const ext::set<Node> &nodes);
	static void composeDirectedEdges(std::ostream &out, const ext::set<DirectedEdge> &edges);
	static void composeUndirectedEdges(std::ostream &out, const ext::set<UndirectedEdge> &edges);

	template<typename T>
	static void composeNodeValues(std::ostream &out, const T &graph);
	template<typename T>
	static void composeEdgeValues(std::ostream &out, const T &graph);

public:
	static void compose(std::ostream& out, const Graph& graph);

	static void compose(std::ostream& out, const DirectedGraph& graph);
	static void compose(std::ostream& out, const UndirectedGraph& graph);
};

} // namespace graph

#endif // GRAPH_TO_STRING_COMPOSER_H_
