/*
 * XmlContainerRegistry.cpp
 *
 *  Created on: 19. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <abstraction/XmlContainerRegistry.hpp>
#include <foreach>

namespace abstraction {

bool XmlContainerRegistry::hasAbstraction ( const std::string & container ) {
	return getEntries ( ).count ( container );
}

std::shared_ptr < abstraction::OperationAbstraction > XmlContainerRegistry::getAbstraction ( const std::string & container, const std::string & paramType, const std::string & filename ) {
	std::set < std::string > paramTypes;
	if ( alib::namingApi::hasTypes ( paramType ) )
		paramTypes = ext::transform < std::string > ( alib::namingApi::getTypes ( paramType ), [ ] ( const ext::type_index & type ) { return ext::to_string ( type ); } );
	else
		paramTypes.insert ( paramType );

	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw exception::CommonException ( "Entry " + container + " not available" );

	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		if ( paramTypes.count ( entry.first ) )
			return entry.second->getAbstraction ( filename );

	throw exception::CommonException ( "Entry for " + container + " parametrized with " + paramType + " not available." );
}

ext::set < std::string > XmlContainerRegistry::listOverloads ( const std::string & container ) {
	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw exception::CommonException ( "Entry " + container + " not available" );

	ext::set < std::string > res;
	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		res.insert ( entry.first );

	return res;
}

ext::set < std::string > XmlContainerRegistry::list ( ) {
	ext::set < std::string > res;

	for ( const std::pair < const std::string, ext::vector < ext::pair < std::string, std::shared_ptr < Entry > > > > & groups : getEntries ( ) )
		res.insert ( groups.first );

	return res;
}

} /* namespace abstraction */
