/*
 * XmlContainerRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _XML_CONTAINER_REGISTRY_HPP_
#define _XML_CONTAINER_REGISTRY_HPP_

#include <functional>
#include <memory>
#include <vector>
#include <string>
#include <set>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

#include <core/namingApi.hpp>

namespace abstraction {

class XmlContainerRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & filename ) const = 0;
	};

	template < class Params >
	class SetEntryImpl : public Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & filename ) const override;
	};

	static ext::map < std::string, ext::vector < ext::pair < std::string, std::shared_ptr < Entry > > > > & getEntries ( ) {
		static ext::map < std::string, ext::vector < ext::pair < std::string, std::shared_ptr < Entry > > > > containerGroups;
		return containerGroups;
	};

public:
	template < class ParamTypes >
	static void registerSet ( std::string param ) {
		std::string container = "Set";

		auto & group = getEntries ( ) [ container ];
		for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group )
			if ( entry.first == param )
				throw exception::CommonException ( "Callback for " + container + " already registered." );

		std::shared_ptr < Entry > entryValue = std::make_shared < SetEntryImpl < ParamTypes > > ( );
		group.push_back ( ext::make_pair ( param, entryValue ) );
	}

	template < class ParamTypes >
	static void registerSet ( ) {
		std::string param = ext::to_string < typename std::decay < ParamTypes >::type > ( );
		registerSet < ParamTypes > ( param );
	}

	static bool hasAbstraction ( const std::string & container );

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & container, const std::string & paramType, const std::string & filename );

	static ext::set < std::string > listOverloads ( const std::string & algorithm );

	static ext::set < std::string > list ( );
};

} /* namespace abstraction */

#include <abstraction/XmlParserAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > XmlContainerRegistry::SetEntryImpl < Param >::getAbstraction ( const std::string & filename ) const {
	ext::deque < sax::Token > tokens = sax::FromXMLParserHelper::parseInput ( filename );
	return std::make_shared < abstraction::XmlParserAbstraction < ext::set < Param > > > ( std::move ( tokens ) );
}

} /* namespace abstraction */

#endif /* _XML_CONTAINER_REGISTRY_HPP_ */
