/*
 * XmlFileWriterRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _XML_FILE_WRITER_REGISTRY_HPP_
#define _XML_FILE_WRITER_REGISTRY_HPP_

#include <memory>
#include <string>
#include <set>
#include <map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class XmlFileWriterRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string filename ) const = 0;

	};

	template < class Param >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string filename ) const override;
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::string, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ParamType >
	static void registerXmlFileWriter ( std::string param ) {
		getEntries ( ).insert ( std::make_pair ( param, std::unique_ptr < Entry > ( new EntryImpl < ParamType > ( ) ) ) );
	}

	template < class ParamType >
	static void registerXmlFileWriter ( ) {
		std::string param = ext::to_string < ParamType > ( );
		registerXmlFileWriter < ParamType > ( std::move ( param ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & param, std::string filename );

	static ext::set < std::string > listGroup ( const std::string & group );

	static ext::set < std::string > list ( );
};

} /* namespace abstraction */

#include <abstraction/XmlFileWriterAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > XmlFileWriterRegistry::EntryImpl < Param >::getAbstraction ( std::string filename ) const {
	return std::make_shared < abstraction::XmlFileWriterAbstraction < const Param & > > ( std::move ( filename ) );
}


} /* namespace abstraction */

#endif /* _XML_FILE_WRITER_REGISTRY_HPP_ */
