/*
 * XmlParserAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _XML_PARSER_ABSTRACTION_HPP_
#define _XML_PARSER_ABSTRACTION_HPP_

#include <abstraction/NullaryOperationAbstraction.hpp>
#include <tuple>
#include <exception/CommonException.h>
#include <factory/XmlDataFactory.hpp>

namespace abstraction {

template < class ReturnType >
class XmlParserAbstraction : public NullaryOperationAbstraction < ReturnType > {
	ext::deque < sax::Token > m_tokens;

public:
	XmlParserAbstraction ( ext::deque < sax::Token > tokens ) : m_tokens ( std::move ( tokens ) ) {
	}

	virtual bool run ( ) override {
		if ( this->isReady ( ) )
			return true;

		ReturnType res = alib::XmlDataFactory::fromTokens ( std::move ( m_tokens ) );
		this->m_data = std::move ( res );
		return true;
	}
};

template < >
inline bool XmlParserAbstraction < exception::CommonException >::run ( ) {
	exception::CommonException ex = alib::XmlDataFactory::fromTokens ( std::move ( m_tokens ) );
	throw ex;
}

} /* namespace abstraction */

#endif /* _XML_PARSER_ABSTRACTION_HPP_ */
