/*
 * Set.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef _XML_OBJECTS_SET_H_
#define _XML_OBJECTS_SET_H_

#include <container/ObjectsSet.h>
#include <core/xmlApi.hpp>

namespace alib {

template < typename T >
struct xmlApi < ext::set < T > > {
	static ext::set < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::set < T > & data );
};

template < typename T >
ext::set < T > xmlApi < ext::set < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	ext::set < T > set;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		set.insert ( alib::xmlApi < T >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return set;
}

template < typename T >
bool xmlApi < ext::set < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename T >
const std::string & xmlApi < ext::set < T > >::xmlTagName ( ) {
	static std::string xmlTagName = "Set";

	return xmlTagName;
}

template < typename T >
void xmlApi < ext::set < T > >::compose ( ext::deque < sax::Token > & output, const ext::set < T > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );

	for ( const T & item : input )
		alib::xmlApi < T >::compose ( output, item );

	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < typename T >
struct xmlApi < container::ObjectsSet < T > > {
	static container::ObjectsSet < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const container::ObjectsSet < T > & data );
};

template < typename T >
container::ObjectsSet < T > xmlApi < container::ObjectsSet < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsSet < T > ( xmlApi < ext::set < T > >::parse ( input ) );
}

template < typename T >
bool xmlApi < container::ObjectsSet < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return xmlApi < ext::set < T > >::first ( input );
}

template < typename T >
const std::string & xmlApi < container::ObjectsSet < T > >::xmlTagName ( ) {
	return xmlApi < ext::set < T > >::xmlTagName ( );
}

template < typename T >
void xmlApi < container::ObjectsSet < T > >::compose ( ext::deque < sax::Token > & output, const container::ObjectsSet < T > & input ) {
	xmlApi < ext::set < T > >::compose ( output, input );
}

} /* namespace alib */

#endif /* _XML_OBJECTS_SET_H_ */
