/*
 * CommonException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "CommonException.h"

#include <object/Object.h>
#include <registration/XmlRegistration.hpp>

namespace alib {

exception::CommonException xmlApi < exception::CommonException >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "cause");
	std::string cause = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		cause = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "cause");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "backtrace");
	std::string backtrace = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		backtrace = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "backtrace");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "command");
	std::string command = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		command = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "command");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, xmlTagName());
	return exception::CommonException(std::move(cause), std::move(backtrace), std::move(command));
}

bool xmlApi < exception::CommonException >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < exception::CommonException >::xmlTagName ( ) {
	static std::string xmlTagName = "CommonException";

	return xmlTagName;
}

void xmlApi < exception::CommonException >::compose ( ext::deque < sax::Token > & output, const exception::CommonException & data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( "cause", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getCause ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "cause", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( "backtrace", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getBacktrace ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "backtrace", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( "command", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getCommand ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "command", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace alib */

namespace {

static auto xmlWrite = registration::XmlWriterRegister < alib::Object, exception::CommonException > ( );

static auto xmlReaded = registration::XmlReaderRegister < alib::Object, exception::CommonException > ( );

} /* namespace */
