/*
 * anormalize.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * * argv ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Automaton normalize binary", ' ', "0.01" );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input to normalize", false, "-", "file" );
		cmd.add ( input );

		std::vector < std::string > forms {
			"leftRG", "rightRG", "singleInitialState", "CNF", "GNF", "LL1"
		};
		TCLAP::ValuesConstraint < std::string > formVals ( forms );

		TCLAP::ValueArg < std::string > form ( "f", "form", "Convert to different form", false, "", & formVals );
		cmd.add ( form );

		std::vector < std::string > inputTypes {
			"automaton", "grammar"
		};
		TCLAP::ValuesConstraint < std::string > inputTypeVals ( inputTypes );

		TCLAP::ValueArg < std::string > labels ( "l", "labels", "Normalize labels", false, "", & inputTypeVals );
		cmd.add ( labels );

		std::vector < std::string > inputTypes2 {
			"string"
		};
		TCLAP::ValuesConstraint < std::string > inputTypeVals2 ( inputTypes2 );

		TCLAP::ValueArg < std::string > alphabet ( "a", "alphabet", "Normalize alphabet", false, "", & inputTypeVals2 );
		cmd.add ( alphabet );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		cli::Parser parser ( cli::Lexer ( "execute <#stdin > $input" ) );
		parser.parse ( )->run ( environment );

		std::string cliCommand;
		if ( labels.getValue ( ) == "automaton" ) {
			cliCommand = "execute automaton::simplify::Normalize $input > $output";
		} else if ( labels.getValue ( ) == "grammar" ) {
			cliCommand = "execute grammar::simplify::Normalize $input > $output";
		} else if ( alphabet.getValue ( ) == "string" ) {
			cliCommand = "execute string::simplify::NormalizeAlphabet $input > $output";
		} else if ( form.getValue ( ) == "leftRG" ) {
			cliCommand = "execute grammar::convert::ToGrammarLeftRG $input > $output";
		} else if ( form.getValue ( ) == "rightRG" ) {
			cliCommand = "execute grammar::convert::ToGrammarRightRG $input > $output";
		} else if ( form.getValue ( ) == "singleInitialState" ) {
			cliCommand = "execute automaton::simplify::SingleInitialState $input > $output";
		} else if ( form.getValue ( ) == "CNF" ) {
			cliCommand = "execute grammar::simplify::ToCNF $input > $output";
		} else if ( form.getValue ( ) == "GNF" ) {
			cliCommand = "execute grammar::simplify::ToGNF $input > $output";
		} else if ( form.getValue ( ) == "LL1" ) {
			cliCommand = "execute grammar::parsing::DeterministicLL1Grammar $input > $output";
		} else {
			throw exception::CommonException ( "Invalid normalize command" );
		}

		parser = cli::Parser ( cli::Lexer ( cliCommand ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		parser = cli::Parser ( cli::Lexer ( "execute $output >#stdout" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
