/*
 * astringology.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Stringology algorithm access binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "exactFactorMatch" );
		allowed.push_back ( "boyerMooreHorspool" );
		allowed.push_back ( "boyerMoore" );
		allowed.push_back ( "reversedBoyerMooreHorspool" );
		allowed.push_back ( "deadZoneUsingBadCharacterShift" );

		allowed.push_back ( "exactMatchingAutomaton" );
		allowed.push_back ( "exactNondeterministicFactorAutomaton" );
		allowed.push_back ( "exactSubsequenceAutomaton" );
		allowed.push_back ( "exactNondeterministicSubsequenceAutomaton" );
		allowed.push_back ( "exactMultiNondeterministicSubsequenceAutomaton" );
		allowed.push_back ( "exactFactorOracleAutomaton" );
		allowed.push_back ( "exactNondeterministicSuffixAutomaton" );
		allowed.push_back ( "bndmOccurrences" );
		allowed.push_back ( "backwardOracleMatching" );
		allowed.push_back ( "backwardDAWGMatching" );
		allowed.push_back ( "suffixTrie" );
		allowed.push_back ( "positionHeap" );
		allowed.push_back ( "bitParallelIndex" );
		allowed.push_back ( "bndmMatcher" );
		allowed.push_back ( "compressedBitParallelIndex" );
		allowed.push_back ( "suffixArray" );

		allowed.push_back ( "borderArray" );
		allowed.push_back ( "badCharacterShiftTable" );
		allowed.push_back ( "goodSuffixShiftTable" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > algorithm ( "a", "algorithm", "Execute algorithm", false, "exactFactorMatch", & allowedVals );
		cmd.add ( algorithm );

		TCLAP::MultiArg < std::string > subjectInput ( "s", "subject", "Subject string from file", false, "file" );
		cmd.add ( subjectInput );

		TCLAP::MultiArg < std::string > patternInput ( "p", "pattern", "Pattern string from file", false, "file" );
		cmd.add ( patternInput );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		if ( algorithm.getValue ( ) == "exactFactorMatch"
		  || algorithm.getValue ( ) == "boyerMooreHorspool"
		  || algorithm.getValue ( ) == "boyerMoore"
		  || algorithm.getValue ( ) == "reversedBoyerMooreHorspool"
		  || algorithm.getValue ( ) == "deadZoneUsingBadCharacterShift"
		  || algorithm.getValue ( ) == "bndmOccurrences"
		  || algorithm.getValue ( ) == "backwardOracleMatching"
		  || algorithm.getValue ( ) == "backwardDAWGMatching"
		  || algorithm.getValue ( ) == "exactNondeterministicFactorAutomaton"
		  || algorithm.getValue ( ) == "exactSubsequenceAutomaton"
		  || algorithm.getValue ( ) == "exactNondeterministicSubsequenceAutomaton"
		  || algorithm.getValue ( ) == "borderArray"
		  || algorithm.getValue ( ) == "suffixTrie"
		  || algorithm.getValue ( ) == "positionHeap"
		  || algorithm.getValue ( ) == "bitParallelIndex"
		  || algorithm.getValue ( ) == "compressedBitParallelIndex"
		  || algorithm.getValue ( ) == "suffixArray" ) {
			std::string input;
			if ( subjectInput.getValue ( ).size ( ) == 0 )
				input = "-";
			else if ( subjectInput.getValue ( ).size ( ) == 1 )
				input = * subjectInput.getValue ( ).begin ( );
			else
				throw exception::CommonException("Multiple parameters when single required.");

			environment.setBinding ( "inputSubject", input );
			cli::Parser parser ( cli::Lexer ( "execute <#inputSubject > $subject" ) );
			parser.parse ( )->run ( environment );
		}

		if ( algorithm.getValue ( ) == "exactFactorMatch"
		  || algorithm.getValue ( ) == "boyerMooreHorspool"
		  || algorithm.getValue ( ) == "boyerMoore"
		  || algorithm.getValue ( ) == "reversedBoyerMooreHorspool"
		  || algorithm.getValue ( ) == "deadZoneUsingBadCharacterShift"
		  || algorithm.getValue ( ) == "bndmOccurrences"
		  || algorithm.getValue ( ) == "backwardOracleMatching"
		  || algorithm.getValue ( ) == "backwardDAWGMatching"
		  || algorithm.getValue ( ) == "exactMatchingAutomaton"
		  || algorithm.getValue ( ) == "exactFactorOracleAutomaton"
		  || algorithm.getValue ( ) == "exactNondeterministicSuffixAutomaton"
		  || algorithm.getValue ( ) == "bndmMatcher"
		  || algorithm.getValue ( ) == "badCharacterShiftTable"
		  || algorithm.getValue ( ) == "goodSuffixShiftTable" ) {
			std::string input;
			if ( patternInput.getValue ( ).size ( ) == 0 )
				input = "-";
			else if ( patternInput.getValue ( ).size ( ) == 1 )
				input = * patternInput.getValue ( ).begin ( );
			else
				throw exception::CommonException("Multiple parameters when single required.");

			environment.setBinding ( "inputPattern", input );
			cli::Parser parser ( cli::Lexer ( "execute <#inputPattern > $pattern" ) );
			parser.parse ( )->run ( environment );
		}

		if ( algorithm.getValue ( ) == "exactMultiNondeterministicSubsequenceAutomaton" ) {
			unsigned i = 0;
			std::string fragment;
			for ( const std::string & input : subjectInput.getValue ( ) ) {
				environment.setBinding ( "inputSubject" + ext::to_string ( i ), input );
				fragment += "<#inputSubject" + ext::to_string ( i ) + " ";
				i++;
			}

			cli::Parser parser ( cli::Lexer ( "execute { :type LinearString " + fragment + " } > $subjects" ) );
			parser.parse ( )->run ( environment );
		}

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		std::string cliCommand;
		if ( algorithm.getValue ( ) == "exactFactorMatch" ) {
			cliCommand = "execute stringology::exact::ExactFactorMatch $subject $pattern > $output";
		} else if ( algorithm.getValue ( ) == "boyerMooreHorspool" ) {
			cliCommand = "execute stringology::exact::BoyerMooreHorspool $subject $pattern > $output";
		} else if ( algorithm.getValue ( ) == "boyerMoore" ) {
			cliCommand = "execute stringology::exact::BoyerMoore $subject $pattern > $output";
		} else if ( algorithm.getValue ( ) == "reversedBoyerMooreHorspool" ) {
			cliCommand = "execute stringology::exact::ReversedBoyerMooreHorspool $subject $pattern > $output";
		} else if ( algorithm.getValue ( ) == "deadZoneUsingBadCharacterShift" ) {
			cliCommand = "execute stringology::exact::DeadZoneUsingBadCharacterShift $subject $pattern > $output";

		} else if ( algorithm.getValue ( ) == "bndmOccurrences" ) {
			cliCommand = "execute stringology::query::BNDMOccurrences $pattern $subject > $output";
		} else if ( algorithm.getValue ( ) == "backwardOracleMatching" ) {
			cliCommand = "execute stringology::query::BackwardOracleMatching $subject $pattern > $output";
		} else if ( algorithm.getValue ( ) == "backwardDAWGMatching" ) {
			cliCommand = "execute stringology::query::BackwardDAWGMatching $subject $pattern > $output";

		} else if ( algorithm.getValue ( ) == "exactNondeterministicFactorAutomaton" ) {
			cliCommand = "execute stringology::indexing::NondeterministicExactFactorAutomaton $subject > $output";
		} else if ( algorithm.getValue ( ) == "exactSubsequenceAutomaton" ) {
			cliCommand = "execute stringology::indexing::ExactSubsequenceAutomaton $subject > $output";
		} else if ( algorithm.getValue ( ) == "exactNondeterministicSubsequenceAutomaton" ) {
			cliCommand = "execute stringology::indexing::ExactNondeterministicSubsequenceAutomaton $subject > $output";
		} else if ( algorithm.getValue ( ) == "exactMultiNondeterministicSubsequenceAutomaton" ) {
			cliCommand = "execute stringology::indexing::ExactMultiNondeterministicSubsequenceAutomaton $subjects > $output";
		} else if ( algorithm.getValue ( ) == "exactFactorOracleAutomaton" ) {
			cliCommand = "execute stringology::indexing::ExactFactorOracleAutomaton $pattern > $output";
		} else if ( algorithm.getValue ( ) == "exactNondeterministicSuffixAutomaton" ) {
			cliCommand = "execute stringology::indexing::NondeterministicExactSuffixAutomaton $pattern > $output";
		} else if ( algorithm.getValue ( ) == "suffixTrie" ) {
			cliCommand = "execute stringology::indexing::SuffixTrieNaive $subject > $output";
		} else if ( algorithm.getValue ( ) == "positionHeap" ) {
			cliCommand = "execute stringology::indexing::PositionHeapNaive $subject > $output";
		} else if ( algorithm.getValue ( ) == "bitParallelIndex" ) {
			cliCommand = "execute stringology::indexing::BitParallelIndexConstruction $subject > $output";
		} else if ( algorithm.getValue ( ) == "compressedBitParallelIndex" ) {
			cliCommand = "execute stringology::indexing::CompressedBitParallelIndexConstruction $subject > $output";
		} else if ( algorithm.getValue ( ) == "suffixArray" ) {
			cliCommand = "execute stringology::indexing::SuffixArrayNaive $subject > $output";

		} else if ( algorithm.getValue ( ) == "bndmMatcher" ) {
			cliCommand = "execute stringology::matching::BNDMMatcherConstruction $pattern > $output";
		} else if ( algorithm.getValue ( ) == "exactMatchingAutomaton" ) {
			cliCommand = "execute stringology::matching::ExactMatchingAutomaton $pattern > $output";

		} else if ( algorithm.getValue ( ) == "borderArray" ) {
			cliCommand = "execute string::properties::BorderArray $subject > $output";
		} else if ( algorithm.getValue ( ) == "badCharacterShiftTable" ) {
			cliCommand = "execute string::properties::BadCharacterShiftTable $pattern > $output";
		} else if ( algorithm.getValue ( ) == "goodSuffixShiftTable" ) {
			cliCommand = "execute string::properties::GoodSuffixShiftTable $pattern > $output";
		} else {
			throw exception::CommonException ( "Invalid algorithm" );
		}

		cli::Parser parser = cli::Parser ( cli::Lexer ( cliCommand ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		parser = cli::Parser ( cli::Lexer ( "execute $output >#stdout" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
